@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2011, 2012, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2011, 2015, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -12,7 +12,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1335  USA
 */
 
 #include <m_string.h>
@@ -101,7 +101,7 @@ int main(int argc,char *argv[])
 
   MY_INIT(argv[0]);
   sf_leaking_memory=1; /* don't report memory leaks on early exits */
-  plugin_data.name= 0; // initialize name
+  plugin_data.name= 0; /* initialize name                          */
   
   /*
     The following operations comprise the method for enabling or disabling
@@ -407,7 +407,7 @@ exit:
 static void usage(void)
 {
   PRINT_VERSION;
-  puts("Copyright (c) 2011, Oracle and/or its affiliates. "
+  puts("Copyright (c) 2011, 2015, Oracle and/or its affiliates. "
        "All rights reserved.\n");
   puts("Enable or disable plugins.");
   printf("\nUsage: %s [options] <plugin> ENABLE|DISABLE\n\nOptions:\n",
@@ -655,11 +655,11 @@ static int load_plugin_data(char *plugin_name, char *config_file)
       }
       break;
     }
-    if ((line[0] == '#') || (line[0] == '\n')) // skip comment and blank lines
+    if ((line[0] == '#') || (line[0] == '\n')) /* skip comment and blank lines */
     {
       continue;
     }
-    if (i == -1) // if first pass, read this line as so_name
+    if (i == -1) /* if first pass, read this line as so_name */
     {
       /* Add proper file extension for soname */
       strcat(line, FN_SOEXT);
@@ -707,10 +707,10 @@ error:
 
 static int check_options(int argc, char **argv, char *operation)
 {
-  int i= 0;                    // loop counter
-  int num_found= 0;            // number of options found (shortcut loop)
-  char config_file[FN_REFLEN]; // configuration file name
-  char plugin_name[FN_REFLEN]; // plugin name
+  int i= 0;                    /* loop counter */
+  int num_found= 0;            /* number of options found (shortcut loop) */
+  char config_file[FN_REFLEN]; /* configuration file name */
+  char plugin_name[FN_REFLEN]; /* plugin name */
   
   /* Form prefix strings for the options. */
   const char *basedir_prefix = "--basedir=";
@@ -758,6 +758,11 @@ static int check_options(int argc, char **argv, char *operation)
     /* read the plugin config file and check for match against argument */
     else
     {
+      if (strlen(argv[i]) + 4 + 1 > FN_REFLEN)
+      {
+        fprintf(stderr, "ERROR: argument is too long.\n");
+        return 1;
+      }
       strcpy(plugin_name, argv[i]);
       strcpy(config_file, argv[i]);
       strcat(config_file, ".ini");
@@ -849,6 +854,7 @@ static int process_options(int argc, char *argv[], char *operation)
     if (opt_basedir[i-1] != FN_LIBCHAR || opt_basedir[i-1] != FN_LIBCHAR2)
     {
       char buff[FN_REFLEN];
+      memset(buff, 0, sizeof(buff));
       
       strncpy(buff, opt_basedir, sizeof(buff) - 1);
 #ifdef __WIN__
@@ -857,7 +863,7 @@ static int process_options(int argc, char *argv[], char *operation)
       strncat(buff, FN_DIRSEP, sizeof(buff) - strlen(buff) - 1);
 #endif
       buff[sizeof(buff) - 1]= 0;
-      my_delete(opt_basedir, MYF(0));
+      my_free(opt_basedir);
       opt_basedir= my_strdup(buff, MYF(MY_FAE));
     }
   }
