@@ -1,6 +1,6 @@
 /*
    Copyright (c) 2006, 2013, Oracle and/or its affiliates.
-   Copyright (c) 2010, 2013, Monty Program Ab.
+   Copyright (c) 2010, 2017, MariaDB
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,7 +13,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1335  USA
 */
 
 #include "client_priv.h"
@@ -22,7 +22,7 @@
 
 #include <welcome_copyright_notice.h> /* ORACLE_WELCOME_COPYRIGHT_NOTICE */
 
-#define VER "1.3"
+#define VER "1.3a"
 
 #ifdef HAVE_SYS_WAIT_H
 #include <sys/wait.h>
@@ -36,6 +36,8 @@
 # endif
 #endif
 
+static int phase = 1;
+static int phases_total = 4;
 static char mysql_path[FN_REFLEN];
 static char mysqlcheck_path[FN_REFLEN];
 
@@ -51,6 +53,8 @@ static DYNAMIC_STRING conn_args;
 static char *opt_password= 0;
 static char *opt_plugin_dir= 0, *opt_default_auth= 0;
 
+static char *cnf_file_path= 0, defaults_file[FN_REFLEN + 32];
+
 static my_bool tty_password= 0;
 
 static char opt_tmpdir[FN_REFLEN] = "";
@@ -107,6 +111,7 @@ static struct my_option my_long_options[]=
    &opt_force, &opt_force, 0, GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0},
   {"host", 'h', "Connect to host.", 0,
    0, 0, GET_STR, REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
+#define PASSWORD_OPT 12
   {"password", 'p',
    "Password to use when connecting to server. If password is not given,"
    " it's solicited on the tty.", &opt_password,&opt_password,
@@ -144,6 +149,7 @@ static struct my_option my_long_options[]=
    "do not try to upgrade the data.",
    &opt_systables_only, &opt_systables_only, 0,
    GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0},
+#define USER_OPT (array_elements(my_long_options) - 6)
   {"user", 'u', "User for login if not current user.", &opt_user,
    &opt_user, 0, GET_STR, REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
   {"verbose", 'v', "Display more output about the process.",
@@ -156,22 +162,33 @@ static struct my_option my_long_options[]=
    "version\' of the program is the version of the MySQL server with which it "
    "was built/distributed.", &opt_version_check, &opt_version_check, 0,
    GET_BOOL, NO_ARG, 1, 0, 0, 0, 0, 0},
-  {"write-binlog", OPT_WRITE_BINLOG,
-   "All commands including mysqlcheck are binlogged. Enabled by default;"
-   "use --skip-write-binlog when commands should not be sent to replication slaves.",
+  {"write-binlog", OPT_WRITE_BINLOG, "All commands including those, "
+   "issued by mysqlcheck, are written to the binary log.",
    &opt_write_binlog, &opt_write_binlog, 0, GET_BOOL, NO_ARG,
-   1, 0, 0, 0, 0, 0},
+   0, 0, 0, 0, 0, 0},
   {0, 0, 0, 0, 0, 0, GET_NO_ARG, NO_ARG, 0, 0, 0, 0, 0, 0}
 };
 
 
+static const char *load_default_groups[]=
+{
+  "client",          /* Read settings how to connect to server */
+  "mysql_upgrade",   /* Read special settings for mysql_upgrade */
+  "client-server",   /* Reads settings common between client & server */
+  "client-mariadb",  /* Read mariadb unique client settings */
+  0
+};
+
 static void free_used_memory(void)
 {
   /* Free memory allocated by 'load_defaults' */
-  free_defaults(defaults_argv);
+  if (defaults_argv)
+    free_defaults(defaults_argv);
 
   dynstr_free(&ds_args);
   dynstr_free(&conn_args);
+  if (cnf_file_path)
+    my_delete(cnf_file_path, MYF(MY_WME));
 }
 
 
@@ -181,6 +198,7 @@ static void die(const char *fmt, ...)
   DBUG_ENTER("die");
 
   /* Print the error message */
+  fflush(stdout);
   va_start(args, fmt);
   if (fmt)
   {
@@ -222,31 +240,32 @@ static void verbose(const char *fmt, ...)
   this way we pass the same arguments on to mysql and mysql_check
 */
 
-static void add_one_option(DYNAMIC_STRING* ds,
-                           const struct my_option *opt,
-                           const char* argument)
-
+static void add_one_option_cmd_line(DYNAMIC_STRING *ds,
+                                    const struct my_option *opt,
+                                    const char* arg)
 {
-  const char* eq= NullS;
-  const char* arg= NullS;
-  if (opt->arg_type != NO_ARG)
+  dynstr_append(ds, "--");
+  dynstr_append(ds, opt->name);
+  if (arg)
   {
-    eq= "=";
-    switch (opt->var_type & GET_TYPE_MASK) {
-    case GET_STR:
-      arg= argument;
-      break;
-    case GET_BOOL:
-      arg= (*(my_bool *)opt->value) ? "1" : "0";
-      break;
-    default:
-      die("internal error at %s: %d",__FILE__, __LINE__);
-    }
+    dynstr_append(ds, "=");
+    dynstr_append_os_quoted(ds, arg, NullS);
   }
-  dynstr_append_os_quoted(ds, "--", opt->name, eq, arg, NullS);
   dynstr_append(ds, " ");
 }
 
+static void add_one_option_cnf_file(DYNAMIC_STRING *ds,
+                                    const struct my_option *opt,
+                                    const char* arg)
+{
+  dynstr_append(ds, opt->name);
+  if (arg)
+  {
+    dynstr_append(ds, "=");
+    dynstr_append_os_quoted(ds, arg, NullS);
+  }
+  dynstr_append(ds, "\n");
+}
 
 static my_bool
 get_one_option(int optid, const struct my_option *opt,
@@ -260,8 +279,11 @@ get_one_option(int optid, const struct my_option *opt,
     printf("%s  Ver %s Distrib %s, for %s (%s)\n",
            my_progname, VER, MYSQL_SERVER_VERSION, SYSTEM_TYPE, MACHINE_TYPE);
     puts(ORACLE_WELCOME_COPYRIGHT_NOTICE("2000"));
-    puts("MariaDB utility for upgrading databases to new MariaDB versions.\n");
+    puts("MariaDB utility for upgrading databases to new MariaDB versions.");
+    print_defaults("my", load_default_groups);
+    puts("");
     my_print_help(my_long_options);
+    my_print_variables(my_long_options);
     die(0);
     break;
 
@@ -274,16 +296,17 @@ get_one_option(int optid, const struct my_option *opt,
   case 'p':
     if (argument == disabled_my_option)
       argument= (char*) "";			/* Don't require password */
-    tty_password= 1;
     add_option= FALSE;
     if (argument)
     {
       /* Add password to ds_args before overwriting the arg with x's */
-      add_one_option(&ds_args, opt, argument);
+      add_one_option_cnf_file(&ds_args, opt, argument);
       while (*argument)
         *argument++= 'x';                       /* Destroy argument */
       tty_password= 0;
     }
+    else
+      tty_password= 1;
     break;
 
   case 't':
@@ -330,18 +353,18 @@ get_one_option(int optid, const struct my_option *opt,
   case OPT_SHARED_MEMORY_BASE_NAME: /* --shared-memory-base-name */
   case OPT_PLUGIN_DIR:                          /* --plugin-dir */
   case OPT_DEFAULT_AUTH:                        /* --default-auth */
-    add_one_option(&conn_args, opt, argument);
+    add_one_option_cmd_line(&conn_args, opt, argument);
     break;
   }
 
   if (add_option)
   {
     /*
-      This is an option that is accpted by mysql_upgrade just so
+      This is an option that is accepted by mysql_upgrade just so
       it can be passed on to "mysql" and "mysqlcheck"
       Save it in the ds_args string
     */
-    add_one_option(&ds_args, opt, argument);
+    add_one_option_cnf_file(&ds_args, opt, argument);
   }
   return 0;
 }
@@ -401,11 +424,8 @@ static int run_tool(char *tool_path, DYNAMIC_STRING *ds_res, ...)
 
   while ((arg= va_arg(args, char *)))
   {
-    /* Options should be os quoted */
-    if (strncmp(arg, "--", 2) == 0)
-      dynstr_append_os_quoted(&ds_cmdline, arg, NullS);
-    else
-      dynstr_append(&ds_cmdline, arg);
+    /* Options should already be os quoted */
+    dynstr_append(&ds_cmdline, arg);
     dynstr_append(&ds_cmdline, " ");
   }
 
@@ -488,6 +508,7 @@ static void find_tool(char *tool_executable_name, const char *tool_name,
   */
   if (run_tool(tool_executable_name,
                &ds_tmp, /* Get output from command, discard*/
+               "--no-defaults",
                "--help",
                "2>&1",
                IF_WIN("> NUL", "> /dev/null"),
@@ -546,8 +567,7 @@ static int run_query(const char *query, DYNAMIC_STRING *ds_res,
 
   ret= run_tool(mysql_path,
                 ds_res,
-                "--no-defaults",
-                ds_args.str,
+                defaults_file,
                 "--database=mysql",
                 "--batch", /* Turns off pager etc. */
                 force ? "--force": "--skip-force",
@@ -725,36 +745,102 @@ static void print_conn_args(const char *tool_name)
 
 static int run_mysqlcheck_upgrade(void)
 {
-  verbose("Phase 2/3: Checking and upgrading tables");
+  int retch;
+  if (opt_systables_only)
+  {
+    verbose("Phase %d/%d: Checking and upgrading tables... Skipped",
+            phase++, phases_total);
+    return 0;
+  }
+  verbose("Phase %d/%d: Checking and upgrading tables", phase++, phases_total);
   print_conn_args("mysqlcheck");
-  return run_tool(mysqlcheck_path,
+  retch= run_tool(mysqlcheck_path,
                   NULL, /* Send output from mysqlcheck directly to screen */
-                  "--no-defaults",
-                  ds_args.str,
+                  defaults_file,
                   "--check-upgrade",
                   "--all-databases",
                   "--auto-repair",
                   !opt_silent || opt_verbose ? "--verbose": "",
                   opt_silent ? "--silent": "",
                   opt_write_binlog ? "--write-binlog" : "--skip-write-binlog",
+                  "2>&1",
                   NULL);
+  return retch;
 }
 
+#define EVENTS_STRUCT_LEN 7000
 
-static int run_mysqlcheck_fixnames(void)
+static my_bool is_mysql()
 {
-  verbose("Phase 1/3: Fixing table and database names");
+  my_bool ret= TRUE;
+  DYNAMIC_STRING ds_events_struct;
+
+  if (init_dynamic_string(&ds_events_struct, NULL,
+                          EVENTS_STRUCT_LEN, EVENTS_STRUCT_LEN))
+    die("Out of memory");
+
+  if (run_query("show create table mysql.event",
+                &ds_events_struct, FALSE) ||
+      strstr(ds_events_struct.str, "IGNORE_BAD_TABLE_OPTIONS") != NULL)
+    ret= FALSE;
+  else
+    verbose("MySQL upgrade detected");
+
+  dynstr_free(&ds_events_struct);
+  return(ret);
+}
+
+static int run_mysqlcheck_views(void)
+{
+  const char *upgrade_views="--process-views=YES";
+  if (is_mysql())
+  {
+    upgrade_views="--process-views=UPGRADE_FROM_MYSQL";
+    verbose("Phase %d/%d: Fixing views from mysql", phase++, phases_total);
+  }
+  else if (opt_systables_only)
+  {
+    verbose("Phase %d/%d: Fixing views... Skipped", phase++, phases_total);
+    return 0;
+  }
+  else
+    verbose("Phase %d/%d: Fixing views", phase++, phases_total);
+
   print_conn_args("mysqlcheck");
   return run_tool(mysqlcheck_path,
                   NULL, /* Send output from mysqlcheck directly to screen */
-                  "--no-defaults",
-                  ds_args.str,
+                  defaults_file,
+                  "--all-databases", "--repair",
+                  upgrade_views,
+                  "--skip-process-tables",
+                  opt_verbose ? "--verbose": "",
+                  opt_silent ? "--silent": "",
+                  opt_write_binlog ? "--write-binlog" : "--skip-write-binlog",
+                  "2>&1",
+                  NULL);
+}
+
+static int run_mysqlcheck_fixnames(void)
+{
+  if (opt_systables_only)
+  {
+    verbose("Phase %d/%d: Fixing table and database names ... Skipped",
+            phase++, phases_total);
+    return 0;
+  }
+  verbose("Phase %d/%d: Fixing table and database names",
+          phase++, phases_total);
+  print_conn_args("mysqlcheck");
+  return run_tool(mysqlcheck_path,
+                  NULL, /* Send output from mysqlcheck directly to screen */
+                  defaults_file,
                   "--all-databases",
                   "--fix-db-names",
                   "--fix-table-names",
                   opt_verbose ? "--verbose": "",
                   opt_silent ? "--silent": "",
                   opt_write_binlog ? "--write-binlog" : "--skip-write-binlog",
+                  "2>&1",
                   NULL);
 }
 
@@ -823,7 +909,8 @@ static int run_sql_fix_privilege_tables(void)
   if (init_dynamic_string(&ds_result, "", 512, 512))
     die("Out of memory");
 
-  verbose("Phase 3/3: Running 'mysql_fix_privilege_tables'...");
+  verbose("Phase %d/%d: Running 'mysql_fix_privilege_tables'",
+          phase++, phases_total);
   run_query(mysql_fix_privilege_tables,
             &ds_result, /* Collect result */
             TRUE);
@@ -856,14 +943,11 @@ static int run_sql_fix_privilege_tables(void)
 }
 
 
-static const char *load_default_groups[]=
+static void print_error(const char *error_msg, DYNAMIC_STRING *output)
 {
-  "client",          /* Read settings how to connect to server */
-  "mysql_upgrade",   /* Read special settings for mysql_upgrade */
-  "client-server",   /* Reads settings common between client & server */
-  "client-mariadb",  /* Read mariadb unique client settings */
-  0
-};
+  fprintf(stderr, "%s\n", error_msg);
+  fprintf(stderr, "%s", output->str);
+}
 
 
 /* Convert the specified version string into the numeric format. */
@@ -896,6 +980,8 @@ static int check_version_match(void)
                 &ds_version, FALSE) ||
       extract_variable_from_show(&ds_version, version_str))
   {
+    print_error("Version check failed. Got the following error when calling "
+                "the 'mysql' command line client", &ds_version);
     dynstr_free(&ds_version);
     return 1;                                   /* Query failed */
   }
@@ -947,25 +1033,30 @@ int main(int argc, char **argv)
   {
     opt_password= get_tty_password(NullS);
     /* add password to defaults file */
-    dynstr_append_os_quoted(&ds_args, "--password=", opt_password, NullS);
-    dynstr_append(&ds_args, " ");
+    add_one_option_cnf_file(&ds_args, &my_long_options[PASSWORD_OPT], opt_password);
+    DBUG_ASSERT(strcmp(my_long_options[PASSWORD_OPT].name, "password") == 0);
   }
   /* add user to defaults file */
-  dynstr_append_os_quoted(&ds_args, "--user=", opt_user, NullS);
-  dynstr_append(&ds_args, " ");
+  add_one_option_cnf_file(&ds_args, &my_long_options[USER_OPT], opt_user);
+  DBUG_ASSERT(strcmp(my_long_options[USER_OPT].name, "user") == 0);
+
+  cnf_file_path= strmov(defaults_file, "--defaults-file=");
+  {
+    int fd= create_temp_file(cnf_file_path, opt_tmpdir[0] ? opt_tmpdir : NULL,
+                             "mysql_upgrade-", O_CREAT | O_WRONLY, MYF(MY_FAE));
+    my_write(fd, USTRING_WITH_LEN( "[client]\n"), MYF(MY_FAE));
+    my_write(fd, (uchar*)ds_args.str, ds_args.length, MYF(MY_FAE));
+    my_close(fd, MYF(0));
+  }
 
   /* Find mysql */
   find_tool(mysql_path, IF_WIN("mysql.exe", "mysql"), self_name);
 
-  if (!opt_systables_only)
-  {
-    /* Find mysqlcheck */
-    find_tool(mysqlcheck_path, IF_WIN("mysqlcheck.exe", "mysqlcheck"), self_name);
-  }
-  else
-  {
-    printf("The --upgrade-system-tables option was used, databases won't be touched.\n");
-  }
+  /* Find mysqlcheck */
+  find_tool(mysqlcheck_path, IF_WIN("mysqlcheck.exe", "mysqlcheck"), self_name);
+
+  if (opt_systables_only && !opt_silent)
+    printf("The --upgrade-system-tables option was used, user tables won't be touched.\n");
 
   /*
     Read the mysql_upgrade_info file to check if mysql_upgrade
@@ -976,7 +1067,7 @@ int main(int argc, char **argv)
     printf("This installation of MySQL is already upgraded to %s, "
            "use --force if you still need to run mysql_upgrade\n",
            MYSQL_SERVER_VERSION);
-    die(NULL);
+    goto end;
   }
 
   if (opt_version_check && check_version_match())
@@ -985,9 +1076,8 @@ int main(int argc, char **argv)
   /*
     Run "mysqlcheck" and "mysql_fix_privilege_tables.sql"
   */
-  if ((!opt_systables_only &&
-       (run_mysqlcheck_fixnames() || run_mysqlcheck_upgrade())) ||
-      run_sql_fix_privilege_tables())
+  if (run_mysqlcheck_views() || run_mysqlcheck_fixnames() ||
+      run_mysqlcheck_upgrade() || run_sql_fix_privilege_tables())
   {
     /*
       The upgrade failed to complete in some way or another,
@@ -1000,8 +1090,8 @@ int main(int argc, char **argv)
   /* Create a file indicating upgrade has been performed */
   create_mysql_upgrade_info_file();
 
+end:
   free_used_memory();
   my_end(my_end_arg);
   exit(0);
 }
-
