@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2000, 2011, Oracle and/or its affiliates
+   Copyright (c) 2000, 2014, Oracle and/or its affiliates
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -14,7 +14,7 @@
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
-   02110-1301  USA */
+   02110-1335  USA */
 
 /*
   Replace strings in textfile
@@ -174,6 +174,7 @@ register char **argv[];
 	break;
       case 'V':
 	version=1;
+        /* fall through */
       case 'I':
       case '?':
 	help=1;					/* Help text written */
@@ -265,7 +266,7 @@ static int insert_pointer_name(reg1 POINTER_ARRAY *pa,char * name)
     if (!(pa->str= (uchar*) my_malloc((uint) (PS_MALLOC-MALLOC_OVERHEAD),
 				     MYF(MY_WME))))
     {
-      my_free(pa->typelib.type_names);
+      my_free((void*) pa->typelib.type_names);
       DBUG_RETURN (-1);
     }
     pa->max_count=(PC_MALLOC-MALLOC_OVERHEAD)/(sizeof(uchar*)+
@@ -327,7 +328,7 @@ static void free_pointer_array(reg1 POINTER_ARRAY *pa)
   if (pa->typelib.count)
   {
     pa->typelib.count=0;
-    my_free(pa->typelib.type_names);
+    my_free((void*) pa->typelib.type_names);
     pa->typelib.type_names=0;
     my_free(pa->str);
   }
@@ -1021,7 +1022,7 @@ FILE *in,*out;
   updated=retain=0;
   reset_buffer();
 
-  while ((error=fill_buffer_retaining(fileno(in),retain)) > 0)
+  while ((error=fill_buffer_retaining(my_fileno(in),retain)) > 0)
   {
     end_of_line=buffer ;
     buffer[bufbytes]=0;			/* Sentinel  */
