@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2005, 2012, Oracle and/or its affiliates
+   Copyright (c) 2005, 2017, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,7 +13,7 @@
    You should have received a copy of the GNU General Public License
    along with this program; see the file COPYING. If not, write to the
    Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
-   MA  02110-1301  USA.
+   MA  02110-1335  USA.
 */
 
 /*  yaSSL source implements all SSL.v3 secification structures.
@@ -109,15 +109,12 @@ void ClientDiffieHellmanPublic::build(SSL& ssl)
     uint keyLength = dhClient.get_agreedKeyLength(); // pub and agree same
 
     alloc(keyLength, true);
-    dhClient.makeAgreement(dhServer.get_publicKey(), keyLength);
+    dhClient.makeAgreement(dhServer.get_publicKey(),
+                           dhServer.get_publicKeyLength());
     c16toa(keyLength, Yc_);
     memcpy(Yc_ + KEY_OFFSET, dhClient.get_publicKey(), keyLength);
 
-    // because of encoding first byte might be zero, don't use it for preMaster
-    if (*dhClient.get_agreedKey() == 0) 
-        ssl.set_preMaster(dhClient.get_agreedKey() + 1, keyLength - 1);
-    else
-        ssl.set_preMaster(dhClient.get_agreedKey(), keyLength);
+    ssl.set_preMaster(dhClient.get_agreedKey(), keyLength);
 }
 
 
@@ -196,9 +193,16 @@ void DH_Server::build(SSL& ssl)
     sha.update(tmp.get_buffer(), tmp.get_size());
     sha.get_digest(&hash[MD5_LEN]);
 
-    if (ssl.getSecurity().get_parms().sig_algo_ == rsa_sa_algo)
+    if (ssl.getSecurity().get_parms().sig_algo_ == rsa_sa_algo) {
         auth->sign(signature_, hash, sizeof(hash),
                    ssl.getCrypto().get_random());
+        // check for rsa signautre fault
+        if (!auth->verify(hash, sizeof(hash), signature_,
+                                              auth->get_signatureLength())) {
+            ssl.SetError(rsaSignFault_error);
+            return;
+        }
+    }
     else {
         auth->sign(signature_, &hash[MD5_LEN], SHA_LEN,
                    ssl.getCrypto().get_random());
@@ -220,18 +224,29 @@ void DH_Server::build(SSL& ssl)
 // read PreMaster secret and decrypt, server side
 void EncryptedPreMasterSecret::read(SSL& ssl, input_buffer& input)
 {
+    if (input.get_error()) {
+        ssl.SetError(bad_input);
+        return;
+    }
+
     const CertManager& cert = ssl.getCrypto().get_certManager();
     RSA rsa(cert.get_privateKey(), cert.get_privateKeyLength(), false);
     uint16 cipherLen = rsa.get_cipherLength();
     if (ssl.isTLS()) {
         byte len[2];
-        input.read(len, sizeof(len));
+        len[0] = input[AUTO];
+        len[1] = input[AUTO];
         ato16(len, cipherLen);
     }
     alloc(cipherLen);
     input.read(secret_, length_);
+    if (input.get_error()) {
+        ssl.SetError(bad_input);
+        return;
+    }
 
     opaque preMasterSecret[SECRET_LEN];
+    memset(preMasterSecret, 0, sizeof(preMasterSecret));
     rsa.decrypt(preMasterSecret, secret_, length_, 
                 ssl.getCrypto().get_random());
 
@@ -277,6 +292,11 @@ void EncryptedPreMasterSecret::alloc(int sz)
 // read client's public key, server side
 void ClientDiffieHellmanPublic::read(SSL& ssl, input_buffer& input)
 {
+    if (input.get_error() || input.get_remaining() < (uint)LENGTH_SZ) {
+        ssl.SetError(bad_input);
+        return;
+    }
+
     DiffieHellman& dh = ssl.useCrypto().use_dh();
 
     uint16 keyLength;
@@ -285,15 +305,20 @@ void ClientDiffieHellmanPublic::read(SSL& ssl, input_buffer& input)
     tmp[1] = input[AUTO];
     ato16(tmp, keyLength);
 
+    if (keyLength < dh.get_agreedKeyLength()/2) {
+        ssl.SetError(bad_input);
+        return;
+    }
+
     alloc(keyLength);
     input.read(Yc_, keyLength);
+    if (input.get_error()) {
+        ssl.SetError(bad_input);
+        return;
+    }
     dh.makeAgreement(Yc_, keyLength); 
 
-    // because of encoding, first byte might be 0, don't use for preMaster 
-    if (*dh.get_agreedKey() == 0) 
-        ssl.set_preMaster(dh.get_agreedKey() + 1, dh.get_agreedKeyLength() - 1);
-    else
-        ssl.set_preMaster(dh.get_agreedKey(), dh.get_agreedKeyLength());
+    ssl.set_preMaster(dh.get_agreedKey(), dh.get_agreedKeyLength());
     ssl.makeMasterSecret();
 }
 
@@ -331,6 +356,10 @@ void ClientDiffieHellmanPublic::alloc(int sz, bool offset)
 // read server's p, g, public key and sig, client side
 void DH_Server::read(SSL& ssl, input_buffer& input)
 {
+    if (input.get_error() || input.get_remaining() < (uint)LENGTH_SZ) {
+        ssl.SetError(bad_input);
+        return;
+    }
     uint16 length, messageTotal = 6; // pSz + gSz + pubSz
     byte tmp[2];
 
@@ -341,6 +370,10 @@ void DH_Server::read(SSL& ssl, input_buffer& input)
     messageTotal += length;
 
     input.read(parms_.alloc_p(length), length);
+    if (input.get_error() || input.get_remaining() < (uint)LENGTH_SZ) {
+        ssl.SetError(bad_input);
+        return;
+    }
 
     // g
     tmp[0] = input[AUTO];
@@ -349,6 +382,10 @@ void DH_Server::read(SSL& ssl, input_buffer& input)
     messageTotal += length;
 
     input.read(parms_.alloc_g(length), length);
+    if (input.get_error() || input.get_remaining() < (uint)LENGTH_SZ) {
+        ssl.SetError(bad_input);
+        return;
+    }
 
     // pub
     tmp[0] = input[AUTO];
@@ -357,20 +394,36 @@ void DH_Server::read(SSL& ssl, input_buffer& input)
     messageTotal += length;
 
     input.read(parms_.alloc_pub(length), length);
+    if (input.get_error() || input.get_remaining() < (uint)LENGTH_SZ) {
+        ssl.SetError(bad_input);
+        return;
+    }
 
     // save message for hash verify
     input_buffer message(messageTotal);
     input.set_current(input.get_current() - messageTotal);
     input.read(message.get_buffer(), messageTotal);
     message.add_size(messageTotal);
+    if (input.get_error() || input.get_remaining() < (uint)LENGTH_SZ) {
+        ssl.SetError(bad_input);
+        return;
+    }
 
     // signature
     tmp[0] = input[AUTO];
     tmp[1] = input[AUTO];
     ato16(tmp, length);
 
+    if (length == 0) {
+        ssl.SetError(bad_input);
+        return;
+    }
     signature_ = NEW_YS byte[length];
     input.read(signature_, length);
+    if (input.get_error()) {
+        ssl.SetError(bad_input);
+        return;
+    }
 
     // verify signature
     byte hash[FINISHED_SZ];
@@ -645,6 +698,10 @@ void HandShakeHeader::Process(input_buffer& input, SSL& ssl)
 {
     ssl.verifyState(*this);
     if (ssl.GetError()) return;
+    if (input.get_error()) {
+        ssl.SetError(bad_input);
+        return;
+    }
     const HandShakeFactory& hsf = ssl.getFactory().getHandShake();
     mySTL::auto_ptr<HandShakeBase> hs(hsf.CreateObject(type_));
     if (!hs.get()) {
@@ -810,8 +867,19 @@ uint16 ChangeCipherSpec::get_length() const
 
 
 // CipherSpec processing handler
-void ChangeCipherSpec::Process(input_buffer&, SSL& ssl)
+void ChangeCipherSpec::Process(input_buffer& input, SSL& ssl)
 {
+    if (input.get_error()) {
+        ssl.SetError(bad_input);
+        return;
+    }
+
+    // detect duplicate change_cipher
+    if (ssl.getSecurity().get_parms().pending_ == false) {
+        ssl.order_error();
+        return;
+    }
+
     ssl.useSecurity().use_parms().pending_ = false;
     if (ssl.getSecurity().get_resuming()) {
         if (ssl.getSecurity().get_parms().entity_ == client_end)
@@ -873,6 +941,11 @@ output_buffer& operator<<(output_buffer& output, const Alert& a)
 // Alert processing handler
 void Alert::Process(input_buffer& input, SSL& ssl)
 {
+    if (input.get_error()) {
+        ssl.SetError(bad_input);
+        return;
+    }
+
     if (ssl.getSecurity().get_parms().pending_ == false)  { // encrypted alert
         int            aSz = get_length();  // alert size already read on input
         opaque         verify[SHA_LEN];
@@ -890,12 +963,19 @@ void Alert::Process(input_buffer& input, SSL& ssl)
 
         if (ssl.getSecurity().get_parms().cipher_type_ == block) {
             int    ivExtra = 0;
+            opaque fill;
 
             if (ssl.isTLSv1_1())
                 ivExtra = ssl.getCrypto().get_cipher().get_blockSize();
             int padSz = ssl.getSecurity().get_parms().encrypt_size_ - ivExtra -
                         aSz - digestSz;
-            input.set_current(input.get_current() + padSz);
+            for (int i = 0; i < padSz; i++) 
+                fill = input[AUTO];
+        }
+
+        if (input.get_error()) {
+            ssl.SetError(bad_input);
+            return;
         }
 
         // verify
@@ -1112,6 +1192,11 @@ static int timing_verify(SSL& ssl, const byte* input, int padLen, int t,
 // Process handler for Data
 void Data::Process(input_buffer& input, SSL& ssl)
 {
+    if (input.get_error()) {
+        ssl.SetError(bad_input);
+        return;
+    }
+
     int msgSz = ssl.getSecurity().get_parms().encrypt_size_;
     int pad   = 0, padSz = 0;
     int ivExtra = 0;
@@ -1154,7 +1239,7 @@ void Data::Process(input_buffer& input, SSL& ssl)
 
     int dataSz = msgSz - ivExtra - digestSz - pad - padSz;
 
-    if (dataSz < 0) {
+    if (dataSz < 0 || dataSz > (MAX_RECORD_SIZE + COMPRESS_EXTRA)) {
         ssl.SetError(bad_input);
         return;
     }
@@ -1180,6 +1265,10 @@ void Data::Process(input_buffer& input, SSL& ssl)
 
     // advance past mac and fill
     input.set_current(input.get_current() + digestSz + pad + padSz);
+    if (input.get_error()) {
+        ssl.SetError(bad_input);
+        return;
+    }
 }
 
 
@@ -1199,22 +1288,38 @@ output_buffer& operator<<(output_buffer& output, const HandShakeBase& hs)
 
 Certificate::Certificate(const x509* cert) : cert_(cert) 
 {
-    set_length(cert_->get_length() + 2 * CERT_HEADER); // list and cert size
+    if (cert)
+      set_length(cert_->get_length() + 2 * CERT_HEADER); // list and cert size
+    else
+      set_length(CERT_HEADER); // total blank cert size, just list header
 }
 
 
 const opaque* Certificate::get_buffer() const
 {
-    return cert_->get_buffer(); 
+    if (cert_)
+      return cert_->get_buffer();
+
+    return NULL;
 }
 
 
 // output operator for Certificate
 output_buffer& operator<<(output_buffer& output, const Certificate& cert)
 {
-    uint sz = cert.get_length() - 2 * CERT_HEADER;
+    uint sz = cert.get_length();
     opaque tmp[CERT_HEADER];
 
+    if ((int)sz > CERT_HEADER)
+      sz -= 2 * CERT_HEADER;  // actual cert, not including headers
+    else {
+      sz = 0;                 // blank cert case
+      c32to24(sz, tmp);
+      output.write(tmp, CERT_HEADER);
+
+      return output;
+    }
+
     c32to24(sz + CERT_HEADER, tmp);
     output.write(tmp, CERT_HEADER);
     c32to24(sz, tmp);
@@ -1228,6 +1333,11 @@ output_buffer& operator<<(output_buffer& output, const Certificate& cert)
 // certificate processing handler
 void Certificate::Process(input_buffer& input, SSL& ssl)
 {
+    if (input.get_error()) {
+        ssl.SetError(bad_input);
+        return;
+    }
+
     CertManager& cm = ssl.useCrypto().use_certManager();
   
     uint32 list_sz;
@@ -1264,9 +1374,11 @@ void Certificate::Process(input_buffer& input, SSL& ssl)
             ssl.SetError(YasslError(bad_input));
             return;
         }
-        x509* myCert;
-        cm.AddPeerCert(myCert = NEW_YS x509(cert_sz));
-        input.read(myCert->use_buffer(), myCert->get_length());
+        if (cert_sz) {
+          x509* myCert;
+          cm.AddPeerCert(myCert = NEW_YS x509(cert_sz));
+          input.read(myCert->use_buffer(), myCert->get_length());
+        }
 
         list_sz -= cert_sz + CERT_HEADER;
     }
@@ -1394,6 +1506,10 @@ input_buffer& operator>>(input_buffer& input, ServerHello& hello)
     
     // Session
     hello.id_len_ = input[AUTO];
+    if (hello.id_len_ > ID_LEN) {
+        input.set_error(); 
+        return input;
+    }
     if (hello.id_len_)
         input.read(hello.session_id_, hello.id_len_);
  
@@ -1434,8 +1550,13 @@ output_buffer& operator<<(output_buffer& output, const ServerHello& hello)
 
 
 // Server Hello processing handler
-void ServerHello::Process(input_buffer&, SSL& ssl)
+void ServerHello::Process(input_buffer& input, SSL& ssl)
 {
+    if (input.get_error()) {
+        ssl.SetError(bad_input);
+        return;
+    }
+
     if (ssl.GetMultiProtocol()) {   // SSLv23 support
         if (ssl.isTLS() && server_version_.minor_ < 1)
             // downgrade to SSLv3
@@ -1457,6 +1578,10 @@ void ServerHello::Process(input_buffer&, SSL& ssl)
         ssl.SetError(badVersion_error);
         return;
     }
+    if (cipher_suite_[0] != 0x00) {
+        ssl.SetError(unknown_cipher);
+        return;
+    }
     ssl.set_pending(cipher_suite_[1]);
     ssl.set_random(random_, server_end);
     if (id_len_)
@@ -1529,8 +1654,12 @@ const opaque* ServerHello::get_random() const
 
 
 // Server Hello Done processing handler
-void ServerHelloDone::Process(input_buffer&, SSL& ssl)
+void ServerHelloDone::Process(input_buffer& input, SSL& ssl)
 {
+    if (input.get_error()) {
+        ssl.SetError(bad_input);
+        return;
+    }
     ssl.useStates().useClient() = serverHelloDoneComplete;
 }
 
@@ -1649,8 +1778,13 @@ output_buffer& operator<<(output_buffer& output, const ClientHello& hello)
 
 
 // Client Hello processing handler
-void ClientHello::Process(input_buffer&, SSL& ssl)
+void ClientHello::Process(input_buffer& input, SSL& ssl)
 {
+    if (input.get_error()) {
+        ssl.SetError(bad_input);
+        return;
+    }
+
     // store version for pre master secret
     ssl.useSecurity().use_connection().chVersion_ = client_version_;
 
@@ -1782,9 +1916,17 @@ output_buffer& operator<<(output_buffer& output, const ServerKeyExchange& sk)
 // Server Key Exchange processing handler
 void ServerKeyExchange::Process(input_buffer& input, SSL& ssl)
 {
+    if (input.get_error()) {
+        ssl.SetError(bad_input);
+        return;
+    }
     createKey(ssl);
     if (ssl.GetError()) return;
     server_key_->read(ssl, input);
+    if (input.get_error()) {
+        ssl.SetError(bad_input);
+        return;
+    }
 
     ssl.useStates().useClient() = serverKeyExchangeComplete;
 }
@@ -1906,27 +2048,32 @@ input_buffer& operator>>(input_buffer& input, CertificateRequest& request)
 {
     // types
     request.typeTotal_ = input[AUTO];
+    if (request.typeTotal_ > CERT_TYPES) {
+        input.set_error();
+        return input;
+    }
     for (int i = 0; i < request.typeTotal_; i++)
         request.certificate_types_[i] = ClientCertificateType(input[AUTO]);
 
-    byte tmp[REQUEST_HEADER];
-    input.read(tmp, sizeof(tmp));
+    byte tmp[2];
+    tmp[0] = input[AUTO];
+    tmp[1] = input[AUTO];
     uint16 sz;
     ato16(tmp, sz);
 
     // authorities
     while (sz) {
         uint16 dnSz;
-        input.read(tmp, sizeof(tmp));
+        tmp[0] = input[AUTO];
+        tmp[1] = input[AUTO];
         ato16(tmp, dnSz);
-        
-        DistinguishedName dn;
-        request.certificate_authorities_.push_back(dn = NEW_YS 
-                                                  byte[REQUEST_HEADER + dnSz]);
-        memcpy(dn, tmp, REQUEST_HEADER);
-        input.read(&dn[REQUEST_HEADER], dnSz);
+       
+        input.set_current(input.get_current() + dnSz);
 
         sz -= dnSz + REQUEST_HEADER;
+
+        if (input.get_error())
+            break;
     }
 
     return input;
@@ -1965,13 +2112,17 @@ output_buffer& operator<<(output_buffer& output,
 
 
 // CertificateRequest processing handler
-void CertificateRequest::Process(input_buffer&, SSL& ssl)
+void CertificateRequest::Process(input_buffer& input, SSL& ssl)
 {
+    if (input.get_error()) {
+        ssl.SetError(bad_input);
+        return;
+    }
     CertManager& cm = ssl.useCrypto().use_certManager();
 
-    // make sure user provided cert and key before sending and using
-    if (cm.get_cert() && cm.get_privateKey())
-        cm.setSendVerify();
+    cm.setSendVerify();
+    if (cm.get_cert() == NULL || cm.get_privateKey() == NULL)
+      cm.setSendBlankCert();  // send blank cert, OpenSSL requires now
 }
 
 
@@ -2012,6 +2163,12 @@ void CertificateVerify::Build(SSL& ssl)
         memcpy(sig.get(), len, VERIFY_HEADER);
         rsa.sign(sig.get() + VERIFY_HEADER, hashes_.md5_, sizeof(Hashes),
                  ssl.getCrypto().get_random());
+        // check for rsa signautre fault
+        if (!rsa.verify(hashes_.md5_, sizeof(Hashes), sig.get() + VERIFY_HEADER,
+                                                      rsa.get_cipherLength())) {
+            ssl.SetError(rsaSignFault_error);
+            return;
+        }
     }
     else {  // DSA
         DSS dss(cert.get_privateKey(), cert.get_privateKeyLength(), false);
@@ -2049,12 +2206,18 @@ output_buffer& CertificateVerify::get(output_buffer& out) const
 input_buffer& operator>>(input_buffer& input, CertificateVerify& request)
 {
     byte tmp[VERIFY_HEADER];
-    input.read(tmp, sizeof(tmp));
+    tmp[0] = input[AUTO];
+    tmp[1] = input[AUTO];
 
     uint16 sz = 0;
     ato16(tmp, sz);
     request.set_length(sz);
 
+    if (sz == 0) {
+        input.set_error();
+        return input;
+    }
+
     request.signature_ = NEW_YS byte[sz];
     input.read(request.signature_, sz);
 
@@ -2073,8 +2236,13 @@ output_buffer& operator<<(output_buffer& output,
 
 
 // CertificateVerify processing handler
-void CertificateVerify::Process(input_buffer&, SSL& ssl)
+void CertificateVerify::Process(input_buffer& input, SSL& ssl)
 {
+    if (input.get_error()) {
+        ssl.SetError(bad_input);
+        return;
+    }
+
     const Hashes&      hashVerify = ssl.getHashes().get_certVerify();
     const CertManager& cert       = ssl.getCrypto().get_certManager();
 
@@ -2113,9 +2281,17 @@ output_buffer& operator<<(output_buffer& output, const ClientKeyExchange& ck)
 // Client Key Exchange processing handler
 void ClientKeyExchange::Process(input_buffer& input, SSL& ssl)
 {
+    if (input.get_error()) {
+        ssl.SetError(bad_input);
+        return;
+    }
     createKey(ssl);
     if (ssl.GetError()) return;
     client_key_->read(ssl, input);
+    if (input.get_error()) {
+        ssl.SetError(bad_input);
+        return;
+    }
 
     if (ssl.getCrypto().get_certManager().verifyPeer())
         build_certHashes(ssl, ssl.useHashes().use_certVerify());
@@ -2202,11 +2378,19 @@ output_buffer& operator<<(output_buffer& output, const Finished& fin)
 // Finished processing handler
 void Finished::Process(input_buffer& input, SSL& ssl)
 {
+    if (input.get_error()) {
+        ssl.SetError(bad_input);
+        return;
+    }
     // verify hashes
     const  Finished& verify = ssl.getHashes().get_verify();
     uint finishedSz = ssl.isTLS() ? TLS_FINISHED_SZ : FINISHED_SZ;
     
     input.read(hashes_.md5_, finishedSz);
+    if (input.get_error()) {
+        ssl.SetError(bad_input);
+        return;
+    }
 
     if (memcmp(&hashes_, &verify.hashes_, finishedSz)) {
         ssl.SetError(verify_error);
@@ -2228,19 +2412,23 @@ void Finished::Process(input_buffer& input, SSL& ssl)
     opaque mac[SHA_LEN];   // max size
     int    digestSz = ssl.getCrypto().get_digest().get_digestSize();
     input.read(mac, digestSz);
+    if (input.get_error()) {
+        ssl.SetError(bad_input);
+        return;
+    }
 
     uint ivExtra = 0;
     if (ssl.getSecurity().get_parms().cipher_type_ == block)
         if (ssl.isTLSv1_1())
             ivExtra = ssl.getCrypto().get_cipher().get_blockSize();
 
+    opaque fill;
     int    padSz = ssl.getSecurity().get_parms().encrypt_size_ - ivExtra -
                      HANDSHAKE_HEADER - finishedSz - digestSz;
-    input.set_current(input.get_current() + padSz);
-
-    // verify mac
-    if (memcmp(mac, verifyMAC, digestSz)) {
-        ssl.SetError(verify_error);
+    for (int i = 0; i < padSz; i++) 
+        fill = input[AUTO];
+    if (input.get_error()) {
+        ssl.SetError(bad_input);
         return;
     }
 
