@@ -1,6 +1,6 @@
 /* Copyright (C) 2007 Google Inc.
-   Copyright (c) 2008 MySQL AB, 2008-2009 Sun Microsystems, Inc.
-   Use is subject to license terms.
+   Copyright (c) 2008, 2013, Oracle and/or its affiliates.
+   Copyright (c) 2011, 2016, MariaDB
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,7 +13,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA */
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1335  USA */
 
 
 #include "semisync_master.h"
@@ -478,13 +478,17 @@ void ReplSemiSyncMaster::remove_slave()
   lock();
   rpl_semi_sync_master_clients--;
 
-  /* If user has chosen not to wait if no semi-sync slave available
-     and the last semi-sync slave exits, turn off semi-sync on master
-     immediately.
-   */
-  if (!rpl_semi_sync_master_wait_no_slave &&
-      rpl_semi_sync_master_clients == 0)
-    switch_off();
+  /* Only switch off if semi-sync is enabled and is on */
+  if (getMasterEnabled() && is_on())
+  {
+    /* If user has chosen not to wait if no semi-sync slave available
+       and the last semi-sync slave exits, turn off semi-sync on master
+       immediately.
+     */
+    if (!rpl_semi_sync_master_wait_no_slave &&
+        rpl_semi_sync_master_clients == 0)
+      switch_off();
+  }
   unlock();
 }
 
@@ -548,7 +552,7 @@ int ReplSemiSyncMaster::reportReplyBinlog(uint32 server_id,
 
   if (need_copy_send_pos)
   {
-    strcpy(reply_file_name_, log_file_name);
+    strmake_buf(reply_file_name_, log_file_name);
     reply_file_pos_ = log_file_pos;
     reply_file_name_inited_ = true;
 
@@ -655,7 +659,7 @@ int ReplSemiSyncMaster::commitTrx(const char* trx_wait_binlog_name,
         if (cmp <= 0)
 	{
           /* This thd has a lower position, let's update the minimum info. */
-          strcpy(wait_file_name_, trx_wait_binlog_name);
+          strmake_buf(wait_file_name_, trx_wait_binlog_name);
           wait_file_pos_ = trx_wait_binlog_pos;
 
           rpl_semi_sync_master_wait_pos_backtraverse++;
@@ -666,7 +670,7 @@ int ReplSemiSyncMaster::commitTrx(const char* trx_wait_binlog_name,
       }
       else
       {
-        strcpy(wait_file_name_, trx_wait_binlog_name);
+        strmake_buf(wait_file_name_, trx_wait_binlog_name);
         wait_file_pos_ = trx_wait_binlog_pos;
         wait_file_name_inited_ = true;
 
@@ -676,15 +680,11 @@ int ReplSemiSyncMaster::commitTrx(const char* trx_wait_binlog_name,
       }
 
       /* Calcuate the waiting period. */
-      unsigned long long diff_nsecs =
-        start_ts.tv_nsec + (unsigned long long)wait_timeout_ * TIME_MILLION;
-      abstime.tv_sec = start_ts.tv_sec;
-      while (diff_nsecs >= TIME_BILLION)
-      {
-        abstime.tv_sec++;
-        diff_nsecs -= TIME_BILLION;
-      }
-      abstime.tv_nsec = (long)diff_nsecs;
+      long diff_secs = (long) (wait_timeout_ / TIME_THOUSAND); 
+      long diff_nsecs = (long) ((wait_timeout_ % TIME_THOUSAND) * TIME_MILLION);
+      long nsecs = start_ts.tv_nsec + diff_nsecs;
+      abstime.tv_sec = start_ts.tv_sec + diff_secs + nsecs/TIME_BILLION;
+      abstime.tv_nsec = nsecs % TIME_BILLION;
       
       /* In semi-synchronous replication, we wait until the binlog-dump
        * thread has received the reply on the relevant binlog segment from the
@@ -887,10 +887,7 @@ int ReplSemiSyncMaster::updateSyncHeader(unsigned char *packet,
    * target, do not request replies from the slave.
    */
   if (!getMasterEnabled() || !is_semi_sync_slave())
-  {
-    sync = false;
     return 0;
-  }
 
   function_enter(kWho);
 
@@ -898,15 +895,12 @@ int ReplSemiSyncMaster::updateSyncHeader(unsigned char *packet,
 
   /* This is the real check inside the mutex. */
   if (!getMasterEnabled())
-  {
-    sync = false;
-    goto l_end;
-  }
+    goto l_end; // sync= false at this point in time
 
   if (is_on())
   {
     /* semi-sync is ON */
-    sync = false;     /* No sync unless a transaction is involved. */
+    /* sync= false; No sync unless a transaction is involved. */
 
     if (reply_file_name_inited_)
     {
