@@ -12,7 +12,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1335  USA */
 
 
 /**
@@ -253,6 +253,8 @@ int opt_sum_query(THD *thd,
   int error= 0;
   DBUG_ENTER("opt_sum_query");
 
+  thd->lex->current_select->min_max_opt_list.empty();
+
   if (conds)
     where_tables= conds->used_tables();
 
@@ -444,7 +446,14 @@ int opt_sum_query(THD *thd,
           item_sum->aggregator_clear();
         }
         else
+        {
           item_sum->reset_and_add();
+          /*
+            Save a reference to the item for possible rollback
+            of the min/max optimizations for this select
+          */
+	  thd->lex->current_select->min_max_opt_list.push_back(item_sum);
+        }
         item_sum->make_const();
         recalc_const_item= 1;
         break;
@@ -479,6 +488,24 @@ int opt_sum_query(THD *thd,
 }
 
 
+/*
+  Check if both item1 and item2 are strings, and item1 has fewer characters 
+  than item2.
+*/
+
+static bool check_item1_shorter_item2(Item *item1, Item *item2)
+{
+  if (item1->cmp_type() == STRING_RESULT &&
+      item2->cmp_type() == STRING_RESULT)
+  {
+    int len1= item1->max_length / item1->collation.collation->mbmaxlen;
+    int len2= item2->max_length / item2->collation.collation->mbmaxlen;
+    return len1 < len2;
+  }
+  return false;  /* When the check is not applicable, it means "not bigger" */
+}
+
+
 /**
   Test if the predicate compares a field with constants.
 
@@ -509,7 +536,7 @@ bool simple_pred(Item_func *func_item, Item **args, bool *inv_order)
       if (!(item= it++))
         return 0;
       args[0]= item->real_item();
-      if (args[0]->max_length < args[1]->max_length)
+      if (check_item1_shorter_item2(args[0], args[1]))
         return 0;
       if (it++)
         return 0;
@@ -544,7 +571,7 @@ bool simple_pred(Item_func *func_item, Item **args, bool *inv_order)
     }
     else
       return 0;
-    if (args[0]->max_length < args[1]->max_length)
+    if (check_item1_shorter_item2(args[0], args[1]))
       return 0;
     break;
   case 3:
@@ -559,7 +586,7 @@ bool simple_pred(Item_func *func_item, Item **args, bool *inv_order)
         if (!item->const_item())
           return 0;
         args[i]= item;
-        if (args[0]->max_length < args[i]->max_length)
+        if (check_item1_shorter_item2(args[0], args[1]))
           return 0;
       }
     }
@@ -1020,6 +1047,7 @@ static int maxmin_in_range(bool max_fl, Field* field, COND *cond)
   case Item_func::LT_FUNC:
   case Item_func::LE_FUNC:
     less_fl= 1;
+    /* fall through */
   case Item_func::GT_FUNC:
   case Item_func::GE_FUNC:
   {
