@@ -1,4 +1,4 @@
-/* Copyright (c) 2000, 2010, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2000, 2013, Oracle and/or its affiliates. All rights reserved.
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -11,7 +11,7 @@
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1335  USA */
 
 #include "sql_priv.h"
 #include "mysqld.h"                             // system_charset_info
@@ -279,6 +279,9 @@ Rpl_filter::parse_filter_rule(const char* spec, Add_filter add)
   int status= 0;
   char *arg, *ptr, *pstr;
 
+  if (!spec)
+    return false;
+  
   if (! (ptr= my_strdup(spec, MYF(MY_WME))))
     return true;
 
