@@ -12,7 +12,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1335  USA */
 
 
 /*
@@ -68,8 +68,10 @@ mysql_handle_derived(LEX *lex, uint phases)
 {
   bool res= FALSE;
   THD *thd= lex->thd;
+  DBUG_ENTER("mysql_handle_derived");
+  DBUG_PRINT("enter", ("phases: 0x%x", phases));
   if (!lex->derived_tables)
-    return FALSE;
+    DBUG_RETURN(FALSE);
 
   lex->thd->derived_tables_processing= TRUE;
 
@@ -88,6 +90,7 @@ mysql_handle_derived(LEX *lex, uint phases)
 	 sl= sl->next_select_in_list())
     {
       TABLE_LIST *cursor= sl->get_table_list();
+      sl->changed_elements|= TOUCHED_SEL_DERIVED;
       /*
         DT_MERGE_FOR_INSERT is not needed for views/derived tables inside
         subqueries. Views and derived tables of subqueries should be
@@ -127,7 +130,7 @@ mysql_handle_derived(LEX *lex, uint phases)
     }
   }
   lex->thd->derived_tables_processing= FALSE;
-  return res;
+  DBUG_RETURN(res);
 }
 
 /*
@@ -166,8 +169,12 @@ mysql_handle_single_derived(LEX *lex, TABLE_LIST *derived, uint phases)
   THD *thd= lex->thd;
   uint8 allowed_phases= (derived->is_merged_derived() ? DT_PHASES_MERGE :
                          DT_PHASES_MATERIALIZE);
+  DBUG_ENTER("mysql_handle_single_derived");
+  DBUG_PRINT("enter", ("phases: 0x%x  allowed: 0x%x  alias: '%s'",
+                       phases, allowed_phases,
+                       (derived->alias ? derived->alias : "<NULL>")));
   if (!lex->derived_tables)
-    return FALSE;
+    DBUG_RETURN(FALSE);
 
   lex->thd->derived_tables_processing= TRUE;
 
@@ -189,37 +196,7 @@ mysql_handle_single_derived(LEX *lex, TABLE_LIST *derived, uint phases)
       break;
   }
   lex->thd->derived_tables_processing= FALSE;
-  return res;
-}
-
-
-/**
-  Run specified phases for derived tables/views in the given list
-
-  @param lex        LEX for this thread
-  @param table_list list of derived tables/view to handle
-  @param phase_map  phases to process tables/views through
-
-  @details
-  This function runs phases specified by the 'phases_map' on derived
-  tables/views found in the 'dt_list' with help of the
-  TABLE_LIST::handle_derived function.
-  'lex' is passed as an argument to the TABLE_LIST::handle_derived.
-
-  @return FALSE ok
-  @return TRUE  error
-*/
-
-bool
-mysql_handle_list_of_derived(LEX *lex, TABLE_LIST *table_list, uint phases)
-{
-  for (TABLE_LIST *tl= table_list; tl; tl= tl->next_local)
-  {
-    if (tl->is_view_or_derived() &&
-        tl->handle_derived(lex, phases))
-      return TRUE;
-  }
-  return FALSE;
+  DBUG_RETURN(res);
 }
 
 
@@ -354,9 +331,25 @@ bool mysql_derived_merge(THD *thd, LEX *lex, TABLE_LIST *derived)
   uint tablenr;
   SELECT_LEX *parent_lex= derived->select_lex;
   Query_arena *arena, backup;
+  DBUG_ENTER("mysql_derived_merge");
+  DBUG_PRINT("enter", ("Alias: '%s'  Unit: %p",
+                       (derived->alias ? derived->alias : "<NULL>"),
+                       derived->get_unit()));
 
   if (derived->merged)
-    return FALSE;
+  {
+
+    DBUG_PRINT("info", ("Irreversibly merged: exit"));
+    DBUG_RETURN(FALSE);
+  }
+
+  if (dt_select->uncacheable & UNCACHEABLE_RAND)
+  {
+    /* There is random function => fall back to materialization. */
+    derived->change_refs_to_fields();
+    derived->set_materialized_derived();
+    DBUG_RETURN(FALSE);
+  }
 
  if (thd->lex->sql_command == SQLCOM_UPDATE_MULTI ||
      thd->lex->sql_command == SQLCOM_DELETE_MULTI)
@@ -379,24 +372,18 @@ bool mysql_derived_merge(THD *thd, LEX *lex, TABLE_LIST *derived)
     if (parent_lex->get_free_table_map(&map, &tablenr))
     {
       /* There is no enough table bits, fall back to materialization. */
-      derived->change_refs_to_fields();
-      derived->set_materialized_derived();
-      goto exit_merge;
+      goto unconditional_materialization;
     }
 
     if (dt_select->leaf_tables.elements + tablenr > MAX_TABLES)
     {
       /* There is no enough table bits, fall back to materialization. */
-      derived->change_refs_to_fields();
-      derived->set_materialized_derived();
-      goto exit_merge;
+      goto unconditional_materialization;
     }
 
     if (dt_select->options & OPTION_SCHEMA_TABLE)
       parent_lex->options |= OPTION_SCHEMA_TABLE;
 
-    parent_lex->cond_count+= dt_select->cond_count;
-
     if (!derived->get_unit()->prepared)
     {
       dt_select->leaf_tables.empty();
@@ -437,6 +424,9 @@ bool mysql_derived_merge(THD *thd, LEX *lex, TABLE_LIST *derived)
   {
     Item *expr= derived->on_expr;
     expr= and_conds(expr, dt_select->join ? dt_select->join->conds : 0);
+    if (expr)
+      expr->top_level_item();
+    
     if (expr && (derived->prep_on_expr || expr != derived->on_expr))
     {
       derived->on_expr= expr;
@@ -461,7 +451,16 @@ bool mysql_derived_merge(THD *thd, LEX *lex, TABLE_LIST *derived)
 exit_merge:
   if (arena)
     thd->restore_active_arena(arena, &backup);
-  return res;
+  DBUG_RETURN(res);
+
+unconditional_materialization:
+  derived->change_refs_to_fields();
+  derived->set_materialized_derived();
+  if (!derived->table || !derived->table->created)
+    res= mysql_derived_create(thd, lex, derived);
+  if (!res)
+    res= mysql_derived_fill(thd, lex, derived);
+  goto exit_merge;
 }
 
 
@@ -486,22 +485,31 @@ exit_merge:
 
 bool mysql_derived_merge_for_insert(THD *thd, LEX *lex, TABLE_LIST *derived)
 {
+  DBUG_ENTER("mysql_derived_merge_for_insert");
+  DBUG_PRINT("enter", ("Alias: '%s'  Unit: %p",
+                       (derived->alias ? derived->alias : "<NULL>"),
+                       derived->get_unit()));
   if (derived->merged_for_insert)
-    return FALSE;
+    DBUG_RETURN(FALSE);
+  if (derived->init_derived(thd, FALSE))
+    DBUG_RETURN(TRUE);
   if (derived->is_materialized_derived())
-    return mysql_derived_prepare(thd, lex, derived);
+    DBUG_RETURN(mysql_derived_prepare(thd, lex, derived));
+  if ((thd->lex->sql_command == SQLCOM_UPDATE_MULTI ||
+       thd->lex->sql_command == SQLCOM_DELETE_MULTI))
+    DBUG_RETURN(FALSE);
   if (!derived->is_multitable())
   {
     if (!derived->single_table_updatable())
-      return derived->create_field_translation(thd);
+      DBUG_RETURN(derived->create_field_translation(thd));
     if (derived->merge_underlying_list)
     {
       derived->table= derived->merge_underlying_list->table;
       derived->schema_table= derived->merge_underlying_list->schema_table;
       derived->merged_for_insert= TRUE;
     }
-  }  
-  return FALSE;
+  }
+  DBUG_RETURN(FALSE);
 }
 
 
@@ -527,6 +535,9 @@ bool mysql_derived_init(THD *thd, LEX *lex, TABLE_LIST *derived)
 {
   SELECT_LEX_UNIT *unit= derived->get_unit();
   DBUG_ENTER("mysql_derived_init");
+  DBUG_PRINT("enter", ("Alias: '%s'  Unit: %p",
+                       (derived->alias ? derived->alias : "<NULL>"),
+                       derived->get_unit()));
 
   // Skip already prepared views/DT
   if (!unit || unit->prepared)
@@ -597,6 +608,9 @@ bool mysql_derived_prepare(THD *thd, LEX *lex, TABLE_LIST *derived)
   SELECT_LEX_UNIT *unit= derived->get_unit();
   DBUG_ENTER("mysql_derived_prepare");
   bool res= FALSE;
+  DBUG_PRINT("enter", ("Alias: '%s'  Unit: %p",
+                       (derived->alias ? derived->alias : "<NULL>"),
+                       derived->get_unit()));
 
   // Skip already prepared views/DT
   if (!unit || unit->prepared ||
@@ -606,12 +620,6 @@ bool mysql_derived_prepare(THD *thd, LEX *lex, TABLE_LIST *derived)
           thd->lex->sql_command == SQLCOM_DELETE_MULTI))))
     DBUG_RETURN(FALSE);
 
-  Query_arena *arena= thd->stmt_arena, backup;
-  if (arena->is_conventional())
-    arena= 0;                                   // For easier test
-  else
-    thd->set_n_backup_active_arena(arena, &backup);
-
   SELECT_LEX *first_select= unit->first_select();
 
   /* prevent name resolving out of derived table */
@@ -619,11 +627,34 @@ bool mysql_derived_prepare(THD *thd, LEX *lex, TABLE_LIST *derived)
   {
     sl->context.outer_context= 0;
     // Prepare underlying views/DT first.
-    sl->handle_derived(lex, DT_PREPARE);
+    if ((res= sl->handle_derived(lex, DT_PREPARE)))
+      goto exit;
+
+    if (derived->outer_join && sl->first_cond_optimization)
+    {
+      /* Mark that table is part of OUTER JOIN and fields may be NULL */
+      for (TABLE_LIST *cursor= (TABLE_LIST*) sl->table_list.first;
+           cursor;
+           cursor= cursor->next_local)
+        cursor->outer_join|= JOIN_TYPE_OUTER;
+    }
   }
 
   unit->derived= derived;
 
+  /*
+    Above cascade call of prepare is important for PS protocol, but after it
+    is called we can check if we really need prepare for this derived
+  */
+  if (derived->merged)
+  {
+    DBUG_PRINT("info", ("Irreversibly merged: exit"));
+    DBUG_RETURN(FALSE);
+  }
+
+
+  derived->fill_me= FALSE;
+
   if (!(derived->derived_result= new select_union))
     DBUG_RETURN(TRUE); // out of memory
 
@@ -700,7 +731,7 @@ exit:
   {
     TABLE *table= derived->table;
     table->derived_select_number= first_select->select_number;
-    table->s->tmp_table= NON_TRANSACTIONAL_TMP_TABLE;
+    table->s->tmp_table= INTERNAL_TMP_TABLE;
 #ifndef NO_EMBEDDED_ACCESS_CHECKS
     if (derived->referencing_view)
       table->grant= derived->grant;
@@ -714,9 +745,11 @@ exit:
     /* Add new temporary table to list of open derived tables */
     table->next= thd->derived_tables;
     thd->derived_tables= table;
+
+    /* If table is used by a left join, mark that any column may be null */
+    if (derived->outer_join)
+      table->maybe_null= 1;
   }
-  if (arena)
-    thd->restore_active_arena(arena, &backup);
   DBUG_RETURN(res);
 }
 
@@ -744,9 +777,18 @@ bool mysql_derived_optimize(THD *thd, LEX *lex, TABLE_LIST *derived)
   SELECT_LEX *save_current_select= lex->current_select;
 
   bool res= FALSE;
+  DBUG_ENTER("mysql_derived_optimize");
+  DBUG_PRINT("enter", ("Alias: '%s'  Unit: %p",
+                       (derived->alias ? derived->alias : "<NULL>"),
+                       derived->get_unit()));
+  if (derived->merged)
+  {
+    DBUG_PRINT("info", ("Irreversibly merged: exit"));
+    DBUG_RETURN(FALSE);
+  }
 
   if (unit->optimized)
-    return FALSE;
+    DBUG_RETURN(FALSE);
   lex->current_select= first_select;
 
   if (unit->is_union())
@@ -786,7 +828,7 @@ bool mysql_derived_optimize(THD *thd, LEX *lex, TABLE_LIST *derived)
   }
 err:
   lex->current_select= save_current_select;
-  return res;
+  DBUG_RETURN(res);
 }
 
 
@@ -808,11 +850,15 @@ err:
 
 bool mysql_derived_create(THD *thd, LEX *lex, TABLE_LIST *derived)
 {
+  DBUG_ENTER("mysql_derived_create");
+  DBUG_PRINT("enter", ("Alias: '%s'  Unit: %p",
+                       (derived->alias ? derived->alias : "<NULL>"),
+                       derived->get_unit()));
   TABLE *table= derived->table;
   SELECT_LEX_UNIT *unit= derived->get_unit();
 
   if (table->created)
-    return FALSE;
+    DBUG_RETURN(FALSE);
   select_union *result= (select_union*)unit->result;
   if (table->s->db_type() == TMP_ENGINE_HTON)
   {
@@ -822,13 +868,13 @@ bool mysql_derived_create(THD *thd, LEX *lex, TABLE_LIST *derived)
                                   &result->tmp_table_param.recinfo,
                                   (unit->first_select()->options |
                                    thd->variables.option_bits | TMP_TABLE_ALL_COLUMNS)))
-      return(TRUE);
+      DBUG_RETURN(TRUE);
   }
   if (open_tmp_table(table))
-    return TRUE;
+    DBUG_RETURN(TRUE);
   table->file->extra(HA_EXTRA_WRITE_CACHE);
   table->file->extra(HA_EXTRA_IGNORE_DUP_KEY);
-  return FALSE;
+  DBUG_RETURN(FALSE);
 }
 
 
@@ -857,11 +903,16 @@ bool mysql_derived_create(THD *thd, LEX *lex, TABLE_LIST *derived)
 
 bool mysql_derived_fill(THD *thd, LEX *lex, TABLE_LIST *derived)
 {
+  Field_iterator_table field_iterator;
   SELECT_LEX_UNIT *unit= derived->get_unit();
   bool res= FALSE;
+  DBUG_ENTER("mysql_derived_fill");
+  DBUG_PRINT("enter", ("Alias: '%s'  Unit: %p",
+                       (derived->alias ? derived->alias : "<NULL>"),
+                       derived->get_unit()));
 
   if (unit->executed && !unit->uncacheable && !unit->describe)
-    return FALSE;
+    DBUG_RETURN(FALSE);
   /*check that table creation passed without problems. */
   DBUG_ASSERT(derived->table && derived->table->created);
   SELECT_LEX *first_select= unit->first_select();
@@ -898,12 +949,32 @@ bool mysql_derived_fill(THD *thd, LEX *lex, TABLE_LIST *derived)
     if (derived_result->flush())
       res= TRUE;
     unit->executed= TRUE;
+
+    if (derived->field_translation)
+    {
+      /* reset translation table to materialized table */
+      field_iterator.set_table(derived->table);
+      for (uint i= 0;
+           !field_iterator.end_of_fields();
+           field_iterator.next(), i= i + 1)
+      {
+        Item *item;
+
+        if (!(item= field_iterator.create_item(thd)))
+        {
+          res= TRUE;
+          break;
+        }
+        thd->change_item_tree(&derived->field_translation[i].item, item);
+      }
+    }
   }
+
   if (res || !lex->describe) 
     unit->cleanup();
   lex->current_select= save_current_select;
 
-  return res;
+  DBUG_RETURN(res);
 }
 
 
@@ -926,15 +997,18 @@ bool mysql_derived_fill(THD *thd, LEX *lex, TABLE_LIST *derived)
 
 bool mysql_derived_reinit(THD *thd, LEX *lex, TABLE_LIST *derived)
 {
+  DBUG_ENTER("mysql_derived_reinit");
+  DBUG_PRINT("enter", ("Alias: '%s'  Unit: %p",
+                       (derived->alias ? derived->alias : "<NULL>"),
+                       derived->get_unit()));
   st_select_lex_unit *unit= derived->get_unit();
 
-  if (derived->table)
-    derived->merged_for_insert= FALSE;
+  derived->merged_for_insert= FALSE;
   unit->unclean();
   unit->types.empty();
   /* for derived tables & PS (which can't be reset by Item_subquery) */
   unit->reinit_exec_mechanism();
   unit->set_thd(thd);
-  return FALSE;
+  DBUG_RETURN(FALSE);
 }
 
