@@ -11,7 +11,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1335 USA */
 
 #include <my_global.h>
 #include <violite.h>
@@ -147,8 +147,9 @@ struct thread_group_t
   
 } MY_ALIGNED(512);
 
-static thread_group_t all_groups[MAX_THREAD_GROUPS];
+static thread_group_t *all_groups;
 static uint group_count;
+static int32 shutdown_group_count;
 
 /**
  Used for printing "pool blocked" message, see
@@ -165,6 +166,7 @@ struct pool_timer_t
   volatile uint64 next_timeout_check;
   int  tick_interval;
   bool shutdown;
+  pthread_t timer_thread_id;
 };
 
 static pool_timer_t pool_timer;
@@ -517,7 +519,7 @@ static void* timer_thread(void *param)
       timer->current_microtime= microsecond_interval_timer();
       
       /* Check stalls in thread groups */
-      for(i=0; i< array_elements(all_groups);i++)
+      for (i= 0; i < threadpool_max_size; i++)
       {
         if(all_groups[i].connection_count)
            check_stall(&all_groups[i]);
@@ -602,12 +604,12 @@ void check_stall(thread_group_t *thread_group)
 
 static void start_timer(pool_timer_t* timer)
 {
-  pthread_t thread_id;
   DBUG_ENTER("start_timer");
   mysql_mutex_init(key_timer_mutex,&timer->mutex, NULL);
   mysql_cond_init(key_timer_cond, &timer->cond, NULL);
   timer->shutdown = false;
-  mysql_thread_create(key_timer_thread,&thread_id, NULL, timer_thread, timer);
+  mysql_thread_create(key_timer_thread, &timer->timer_thread_id, NULL,
+                      timer_thread, timer);
   DBUG_VOID_RETURN;
 }
 
@@ -619,6 +621,7 @@ static void stop_timer(pool_timer_t *timer)
   timer->shutdown = true;
   mysql_cond_signal(&timer->cond);
   mysql_mutex_unlock(&timer->mutex);
+  pthread_join(timer->timer_thread_id, NULL);
   DBUG_VOID_RETURN;
 }
 
@@ -907,6 +910,7 @@ int thread_group_init(thread_group_t *thread_group, pthread_attr_t* thread_attr)
   thread_group->pollfd= -1;
   thread_group->shutdown_pipe[0]= -1;
   thread_group->shutdown_pipe[1]= -1;
+  thread_group->queue.empty();
   DBUG_RETURN(0);
 }
 
@@ -927,6 +931,8 @@ void thread_group_destroy(thread_group_t *thread_group)
       thread_group->shutdown_pipe[i]= -1;
     }
   }
+  if (my_atomic_add32(&shutdown_group_count, -1) == 1)
+    my_free(all_groups);
 }
 
 /**
@@ -973,24 +979,26 @@ static void thread_group_close(thread_group_t *thread_group)
 
   if (pipe(thread_group->shutdown_pipe))
   {
-    DBUG_VOID_RETURN;
+    goto end;
   }
   
   /* Wake listener */
   if (io_poll_associate_fd(thread_group->pollfd, 
       thread_group->shutdown_pipe[0], NULL))
   {
-    DBUG_VOID_RETURN;
+    goto end;
+  }
+  {
+    char c= 0;
+    if (write(thread_group->shutdown_pipe[1], &c, 1) < 0)
+      goto end;
   }
-  char c= 0;
-  if (write(thread_group->shutdown_pipe[1], &c, 1) < 0)
-    DBUG_VOID_RETURN;
-
   /* Wake all workers. */
   while(wake_thread(thread_group) == 0) 
   { 
   }
   
+end:
   mysql_mutex_unlock(&thread_group->mutex);
 
   DBUG_VOID_RETURN;
@@ -1165,7 +1173,7 @@ void wait_begin(thread_group_t *thread_group)
   DBUG_ASSERT(thread_group->connection_count > 0);
  
   if ((thread_group->active_thread_count == 0) && 
-     (thread_group->queue.is_empty() || !thread_group->listener))
+     (!thread_group->queue.is_empty() || !thread_group->listener))
   {
     /* 
       Group might stall while this thread waits, thus wake 
@@ -1510,10 +1518,18 @@ static void *worker_main(void *param)
 bool tp_init()
 {
   DBUG_ENTER("tp_init");
+  threadpool_max_size= max(threadpool_size, 128);
+  all_groups= (thread_group_t *)
+    my_malloc(sizeof(thread_group_t) * threadpool_max_size, MYF(MY_WME|MY_ZEROFILL));
+  if (!all_groups)
+  {
+    threadpool_max_size= 0;
+    DBUG_RETURN(1);
+  }
   threadpool_started= true;
   scheduler_init();
 
-  for(uint i=0; i < array_elements(all_groups); i++)
+  for (uint i= 0; i < threadpool_max_size; i++)
   {
     thread_group_init(&all_groups[i], get_connection_attrib());  
   }
@@ -1542,7 +1558,8 @@ void tp_end()
     DBUG_VOID_RETURN;
 
   stop_timer(&pool_timer);
-  for(uint i=0; i< array_elements(all_groups); i++)
+  shutdown_group_count= threadpool_max_size;
+  for (uint i= 0; i < threadpool_max_size; i++)
   {
     thread_group_close(&all_groups[i]);
   }
@@ -1604,9 +1621,7 @@ void tp_set_threadpool_stall_limit(uint limit)
 int tp_get_idle_thread_count()
 {
   int sum=0;
-  for(uint i= 0; 
-      i< array_elements(all_groups) && (all_groups[i].pollfd >= 0); 
-      i++)
+  for (uint i= 0; i < threadpool_max_size && all_groups[i].pollfd >= 0; i++)
   {
     sum+= (all_groups[i].thread_count - all_groups[i].active_thread_count);
   }
