@@ -1,4 +1,5 @@
-/* Copyright (c) 2004, 2011, Oracle and/or its affiliates.
+/* Copyright (c) 2004, 2013, Oracle and/or its affiliates.
+   Copyright (c) 2010, 2014, SkySQL Ab.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -11,7 +12,7 @@
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
-  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
+  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1335  USA */
 
 /**
   @file ha_example.cc
@@ -97,6 +98,7 @@
 #pragma implementation        // gcc: Class implementation
 #endif
 
+#include <my_config.h>
 #include <mysql/plugin.h>
 #include "ha_example.h"
 #include "sql_class.h"
@@ -1057,6 +1059,7 @@ struct st_mysql_storage_engine example_storage_engine=
 
 static ulong srv_enum_var= 0;
 static ulong srv_ulong_var= 0;
+static double srv_double_var= 0;
 
 const char *enum_var_names[]=
 {
@@ -1079,6 +1082,9 @@ static MYSQL_SYSVAR_ENUM(
   0,                              // def
   &enum_var_typelib);             // typelib
 
+static MYSQL_THDVAR_INT(int_var, PLUGIN_VAR_RQCMDARG, "-1..1",
+  NULL, NULL, 0, -1, 1, 0);
+
 static MYSQL_SYSVAR_ULONG(
   ulong_var,
   srv_ulong_var,
@@ -1091,9 +1097,35 @@ static MYSQL_SYSVAR_ULONG(
   1000,
   0);
 
+static MYSQL_SYSVAR_DOUBLE(
+  double_var,
+  srv_double_var,
+  PLUGIN_VAR_RQCMDARG,
+  "0.500000..1000.500000",
+  NULL,
+  NULL,
+  8.5,
+  0.5,
+  1000.5,
+  0);                             // reserved always 0
+
+static MYSQL_THDVAR_DOUBLE(
+  double_thdvar,
+  PLUGIN_VAR_RQCMDARG,
+  "0.500000..1000.500000",
+  NULL,
+  NULL,
+  8.5,
+  0.5,
+  1000.5,
+  0);
+
 static struct st_mysql_sys_var* example_system_variables[]= {
   MYSQL_SYSVAR(enum_var),
   MYSQL_SYSVAR(ulong_var),
+  MYSQL_SYSVAR(int_var),
+  MYSQL_SYSVAR(double_var),
+  MYSQL_SYSVAR(double_thdvar),
   NULL
 };
 
@@ -1104,8 +1136,10 @@ static int show_func_example(MYSQL_THD thd, struct st_mysql_show_var *var,
   var->type= SHOW_CHAR;
   var->value= buf; // it's of SHOW_VAR_FUNC_BUFF_SIZE bytes
   my_snprintf(buf, SHOW_VAR_FUNC_BUFF_SIZE,
-              "enum_var is %lu, ulong_var is %lu, %.6b", // %b is MySQL extension
-              srv_enum_var, srv_ulong_var, "really");
+              "enum_var is %lu, ulong_var is %lu, int_var is %d, "
+              "double_var is %f, %.6b", // %b is a MySQL extension
+              srv_enum_var, srv_ulong_var, THDVAR(thd, int_var),
+              srv_double_var, "really");
   return 0;
 }
 
