@@ -1,7 +1,8 @@
 /*****************************************************************************
 
-Copyright (c) 1995, 2011, Oracle and/or its affiliates. All Rights Reserved.
+Copyright (c) 1995, 2015, Oracle and/or its affiliates. All Rights Reserved.
 Copyright (c) 2008, Google Inc.
+Copyright (c) 2013, 2014, MariaDB Corporation. All Rights Reserved.
 
 Portions of this file contain modifications contributed and copyrighted by
 Google, Inc. Those modifications are gratefully acknowledged and are described
@@ -18,8 +19,8 @@ ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License along with
-this program; if not, write to the Free Software Foundation, Inc., 59 Temple
-Place, Suite 330, Boston, MA 02111-1307 USA
+this program; if not, write to the Free Software Foundation, Inc., 
+51 Franklin St, Fifth Floor, Boston, MA 02110-1335 USA
 
 *****************************************************************************/
 
@@ -466,11 +467,13 @@ static
 void
 sync_array_cell_print(
 /*==================*/
-	FILE*		file,	/*!< in: file where to print */
-	sync_cell_t*	cell)	/*!< in: sync cell */
+	FILE*		file,		/*!< in: file where to print */
+	sync_cell_t*	cell,		/*!< in: sync cell */
+	os_thread_id_t* reserver)	/*!< out: write reserver or
+					0 */
 {
-	mutex_t*	mutex;
-	rw_lock_t*	rwlock;
+	mutex_t*	mutex = NULL;
+	rw_lock_t*	rwlock = NULL;
 	ulint		type;
 	ulint		writer;
 
@@ -525,6 +528,8 @@ sync_array_cell_print(
 				writer == RW_LOCK_EX
 				? " exclusive\n"
 				: " wait exclusive\n");
+
+			*reserver = rwlock->writer_thread;
 		}
 
 		fprintf(file,
@@ -548,7 +553,6 @@ sync_array_cell_print(
 	}
 }
 
-#ifdef UNIV_SYNC_DEBUG
 /******************************************************************//**
 Looks for a cell with the given thread id.
 @return	pointer to cell or NULL if not found */
@@ -576,6 +580,8 @@ sync_array_find_thread(
 	return(NULL);	/* Not found */
 }
 
+#ifdef UNIV_SYNC_DEBUG
+
 /******************************************************************//**
 Recursion step for deadlock detection.
 @return	TRUE if deadlock detected */
@@ -785,6 +791,7 @@ sync_arr_cell_can_wake_up(
 
 		lock = cell->wait_object;
 
+                os_rmb;
 		if (lock->lock_word > 0) {
 		/* Either unlocked or only read locked. */
 
@@ -796,6 +803,7 @@ sync_arr_cell_can_wake_up(
 		lock = cell->wait_object;
 
                 /* lock_word == 0 means all readers have left */
+                os_rmb;
 		if (lock->lock_word == 0) {
 
 			return(TRUE);
@@ -804,6 +812,7 @@ sync_arr_cell_can_wake_up(
 		lock = cell->wait_object;
 
                 /* lock_word > 0 means no writer or reserved writer */
+                os_rmb;
 		if (lock->lock_word > 0) {
 
 			return(TRUE);
@@ -947,6 +956,7 @@ sync_array_print_long_waits(
 
 		double	diff;
 		void*	wait_object;
+		os_thread_id_t reserver=0;
 
 		cell = sync_array_get_nth_cell(sync_primary_wait_array, i);
 
@@ -962,7 +972,7 @@ sync_array_print_long_waits(
 		if (diff > SYNC_ARRAY_TIMEOUT) {
 			fputs("InnoDB: Warning: a long semaphore wait:\n",
 			      stderr);
-			sync_array_cell_print(stderr, cell);
+			sync_array_cell_print(stderr, cell, &reserver);
 			noticed = TRUE;
 		}
 
@@ -977,6 +987,46 @@ sync_array_print_long_waits(
 		}
 	}
 
+	/* We found a long semaphore wait, wait all threads that are
+	waiting for a semaphore. */
+	if (noticed) {
+		for (i = 0; i < sync_primary_wait_array->n_cells; i++) {
+			void*	wait_object;
+			os_thread_id_t reserver=0;
+
+			cell = sync_array_get_nth_cell(sync_primary_wait_array, i);
+
+			wait_object = cell->wait_object;
+
+			if (wait_object == NULL || !cell->waiting) {
+
+				continue;
+			}
+
+			fputs("InnoDB: Warning: semaphore wait:\n",
+			      stderr);
+			sync_array_cell_print(stderr, cell, &reserver);
+			noticed = TRUE;
+
+			/* Try to output cell information for writer recursive way */
+			while (reserver != 0) {
+				sync_cell_t* reserver_wait;
+
+				reserver_wait = sync_array_find_thread(sync_primary_wait_array, reserver);
+
+				if (reserver_wait &&
+					reserver_wait->wait_object != NULL &&
+					reserver_wait->waiting) {
+					fputs("InnoDB: Warning: Writer thread is waiting this semaphore:\n",
+						stderr);
+					sync_array_cell_print(stderr, reserver_wait, &reserver);
+				} else {
+					reserver = 0;
+				}
+			}
+		}
+	}
+
 	sync_array_exit(sync_primary_wait_array);
 
 	if (noticed) {
@@ -1025,10 +1075,12 @@ sync_array_output_info(
 	sync_cell_t*	cell;
 	ulint		count;
 	ulint		i;
+	os_thread_id_t  r;
 
 	fprintf(file,
-		"OS WAIT ARRAY INFO: reservation count %ld, signal count %ld\n",
-						(long) arr->res_count, (long) arr->sg_count);
+		"OS WAIT ARRAY INFO: reservation count " ULINTPF
+		", signal count " ULINTPF "\n",
+		arr->res_count, arr->sg_count);
 	i = 0;
 	count = 0;
 
@@ -1036,9 +1088,9 @@ sync_array_output_info(
 
 		cell = sync_array_get_nth_cell(arr, i);
 
-	if (cell->wait_object != NULL) {
-		count++;
-			sync_array_cell_print(file, cell);
+		if (cell->wait_object != NULL) {
+			count++;
+			sync_array_cell_print(file, cell, &r);
 		}
 
 		i++;
@@ -1060,3 +1112,66 @@ sync_array_print_info(
 
 	sync_array_exit(arr);
 }
+
+/**********************************************************************//**
+Prints info of the wait array without using any mutexes/semaphores. */
+UNIV_INTERN
+void
+sync_array_print_innodb(void)
+/*=========================*/
+{
+	ulint i;
+	sync_array_t*	arr = sync_array_get();
+
+	fputs("InnoDB: Semaphore wait debug output started for InnoDB:\n", stderr);
+
+	for (i = 0; i < arr->n_cells; i++) {
+		void*	wait_object;
+		sync_cell_t*	cell;
+		os_thread_id_t reserver=(os_thread_id_t)ULINT_UNDEFINED;
+		ulint loop=0;
+
+		cell = sync_array_get_nth_cell(arr, i);
+
+		wait_object = cell->wait_object;
+
+		if (wait_object == NULL || !cell->waiting) {
+
+			continue;
+		}
+
+		fputs("InnoDB: Warning: semaphore wait:\n",
+			      stderr);
+		sync_array_cell_print(stderr, cell, &reserver);
+
+		/* Try to output cell information for writer recursive way */
+		while (reserver != (os_thread_id_t)ULINT_UNDEFINED) {
+			sync_cell_t* reserver_wait;
+
+			reserver_wait = sync_array_find_thread(arr, reserver);
+
+			if (reserver_wait &&
+				reserver_wait->wait_object != NULL &&
+				reserver_wait->waiting) {
+				fputs("InnoDB: Warning: Writer thread is waiting this semaphore:\n",
+					stderr);
+				sync_array_cell_print(stderr, reserver_wait, &reserver);
+
+				if (reserver_wait->thread == reserver) {
+					reserver = (os_thread_id_t)ULINT_UNDEFINED;
+				}
+			} else {
+				reserver = (os_thread_id_t)ULINT_UNDEFINED;
+			}
+
+			/* This is protection against loop */
+			if (loop > 100) {
+				fputs("InnoDB: Warning: Too many waiting threads.\n", stderr);
+				break;
+			}
+		}
+	}
+
+	fputs("InnoDB: Semaphore wait debug output ended:\n", stderr);
+
+}
