@@ -11,7 +11,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1335 USA */
 
 /*
   Creates a index for a database by reading keys, sorting them and outputing
@@ -19,6 +19,7 @@
 */
 
 #include "ma_fulltext.h"
+#include <my_check_opt.h>
 #if defined(MSDOS) || defined(__WIN__)
 #include <fcntl.h>
 #else
@@ -85,6 +86,27 @@ static int write_merge_key_varlen(MARIA_SORT_PARAM *info,
 static inline int
 my_var_write(MARIA_SORT_PARAM *info, IO_CACHE *to_file, uchar *bufs);
 
+/*
+  Sets the appropriate read and write methods for the MARIA_SORT_PARAM
+  based on the variable length key flag.
+*/
+static void set_sort_param_read_write(MARIA_SORT_PARAM *sort_param)
+{
+  if (sort_param->keyinfo->flag & HA_VAR_LENGTH_KEY)
+  {
+    sort_param->write_keys=     write_keys_varlen;
+    sort_param->read_to_buffer= read_to_buffer_varlen;
+    sort_param->write_key=      write_merge_key_varlen;
+  }
+  else
+  {
+    sort_param->write_keys=     write_keys;
+    sort_param->read_to_buffer= read_to_buffer;
+    sort_param->write_key=      write_merge_key;
+  }
+}
+
+
 /*
   Creates a index of sorted keys
 
@@ -114,18 +136,7 @@ int _ma_create_index_by_sort(MARIA_SORT_PARAM *info, my_bool no_messages,
                       (ulong) sortbuff_size, info->key_length,
                       (ulong) info->sort_info->max_records));
 
-  if (info->keyinfo->flag & HA_VAR_LENGTH_KEY)
-  {
-    info->write_keys= write_keys_varlen;
-    info->read_to_buffer=read_to_buffer_varlen;
-    info->write_key=write_merge_key_varlen;
-  }
-  else
-  {
-    info->write_keys= write_keys;
-    info->read_to_buffer=read_to_buffer;
-    info->write_key=write_merge_key;
-  }
+  set_sort_param_read_write(info);
 
   my_b_clear(&tempfile);
   my_b_clear(&tempfile_for_exceptions);
@@ -322,181 +333,182 @@ err:
 } /* find_all_keys */
 
 
+static my_bool _ma_thr_find_all_keys_exec(MARIA_SORT_PARAM* sort_param)
+{
+  int error= 0;
+  ulonglong memavl, old_memavl;
+  uint UNINIT_VAR(keys), idx;
+  uint sort_length;
+  uint maxbuffer;
+  uchar **sort_keys= NULL;
+  DBUG_ENTER("_ma_thr_find_all_keys");
+  DBUG_PRINT("enter", ("master: %d", sort_param->master));
+
+  if (sort_param->sort_info->got_error)
+    DBUG_RETURN(TRUE);
+
+  set_sort_param_read_write(sort_param);
+
+
+  my_b_clear(&sort_param->tempfile);
+  my_b_clear(&sort_param->tempfile_for_exceptions);
+  bzero((char*) &sort_param->buffpek, sizeof(sort_param->buffpek));
+  bzero((char*) &sort_param->unique,  sizeof(sort_param->unique));
+
+  memavl=        max(sort_param->sortbuff_size, MIN_SORT_MEMORY);
+  idx=           sort_param->sort_info->max_records;
+  sort_length=   sort_param->key_length;
+  maxbuffer=     1;
+
+  while (memavl >= MIN_SORT_MEMORY)
+  {
+    if ((my_off_t) (idx+1)*(sort_length+sizeof(char*)) <= (my_off_t) memavl)
+      keys= idx+1;
+    else
+    {
+      ulong skr;
+      do
+      {
+        skr= maxbuffer;
+        if (memavl < sizeof(BUFFPEK)*maxbuffer ||
+            (keys=(memavl-sizeof(BUFFPEK)*maxbuffer)/
+             (sort_length+sizeof(char*))) <= 1 ||
+            keys < maxbuffer)
+        {
+          _ma_check_print_error(sort_param->sort_info->param,
+                                "aria_sort_buffer_size is too small.");
+          goto err;
+        }
+      }
+      while ((maxbuffer= (int) (idx/(keys-1)+1)) != skr);
+    }
+    if ((sort_keys= (uchar **)
+         my_malloc(keys*(sort_length+sizeof(char*))+
+                   ((sort_param->keyinfo->flag & HA_FULLTEXT) ?
+                    HA_FT_MAXBYTELEN : 0), MYF(0))))
+    {
+      if (my_init_dynamic_array(&sort_param->buffpek, sizeof(BUFFPEK),
+                                maxbuffer, maxbuffer / 2))
+      {
+        my_free(sort_keys);
+        sort_keys= NULL;          /* Safety against double free on error. */
+      }
+      else
+        break;
+    }
+    old_memavl= memavl;
+    if ((memavl= memavl/4*3) < MIN_SORT_MEMORY &&
+        old_memavl > MIN_SORT_MEMORY)
+      memavl= MIN_SORT_MEMORY;
+  }
+
+  if (memavl < MIN_SORT_MEMORY)
+  {
+    /* purecov: begin inspected */
+    _ma_check_print_error(sort_param->sort_info->param,
+                          "aria_sort_buffer_size is too small.");
+    my_errno= ENOMEM;
+    goto err;
+    /* purecov: end inspected */
+  }
+
+  if (sort_param->sort_info->param->testflag & T_VERBOSE)
+    printf("Key %d - Allocating buffer for %llu keys\n",
+           sort_param->key + 1, (ulonglong) keys);
+  sort_param->sort_keys= sort_keys;
+
+  idx= error= 0;
+  sort_keys[0]= (uchar*) (sort_keys+keys);
+
+  DBUG_PRINT("info", ("reading keys"));
+  while (!(error= sort_param->sort_info->got_error) &&
+         !(error= (*sort_param->key_read)(sort_param, sort_keys[idx])))
+  {
+    if (sort_param->real_key_length > sort_param->key_length)
+    {
+      if (write_key(sort_param, sort_keys[idx],
+                    &sort_param->tempfile_for_exceptions))
+        goto err;
+      continue;
+    }
+
+    if (++idx == keys)
+    {
+      if (sort_param->write_keys(sort_param, sort_keys, idx - 1,
+                                 (BUFFPEK *)alloc_dynamic(&sort_param->buffpek),
+                                 &sort_param->tempfile))
+        goto err;
+      sort_keys[0]= (uchar*) (sort_keys+keys);
+      memcpy(sort_keys[0], sort_keys[idx - 1], (size_t) sort_param->key_length);
+      idx= 1;
+    }
+    sort_keys[idx]= sort_keys[idx - 1] + sort_param->key_length;
+  }
+  if (error > 0)
+    goto err;
+  if (sort_param->buffpek.elements)
+  {
+    if (sort_param->write_keys(sort_param,sort_keys, idx,
+                               (BUFFPEK *) alloc_dynamic(&sort_param->buffpek),
+                               &sort_param->tempfile))
+      goto err;
+    sort_param->keys= (sort_param->buffpek.elements - 1) * (keys - 1) + idx;
+  }
+  else
+    sort_param->keys= idx;
+
+  sort_param->sort_keys_length= keys;
+  DBUG_RETURN(FALSE);
+
+err:
+  DBUG_PRINT("error", ("got some error"));
+  my_free(sort_keys);
+  sort_param->sort_keys= 0;
+  delete_dynamic(& sort_param->buffpek);
+  close_cached_file(&sort_param->tempfile);
+  close_cached_file(&sort_param->tempfile_for_exceptions);
+
+  DBUG_RETURN(TRUE);
+}
+
 /* Search after all keys and place them in a temp. file */
 
 pthread_handler_t _ma_thr_find_all_keys(void *arg)
 {
   MARIA_SORT_PARAM *sort_param= (MARIA_SORT_PARAM*) arg;
-  int error;
-  size_t memavl,old_memavl;
-  uint sort_length;
-  ulong idx, maxbuffer, keys;
-  uchar **sort_keys=0;
+  my_bool error= FALSE;
+  /* If my_thread_init fails */
+  if (my_thread_init() || _ma_thr_find_all_keys_exec(sort_param))
+    error= TRUE;
 
-  LINT_INIT(keys);
+  /*
+     Thread must clean up after itself.
+  */
+  free_root(&sort_param->wordroot, MYF(0));
+  /*
+     Detach from the share if the writer is involved. Avoid others to
+     be blocked. This includes a flush of the write buffer. This will
+     also indicate EOF to the readers.
+     That means that a writer always gets here first and readers -
+     only when they see EOF. But if a reader finishes prematurely
+     because of an error it may reach this earlier - don't allow it
+     to detach the writer thread.
+     */
+  if (sort_param->master && sort_param->sort_info->info->rec_cache.share)
+    remove_io_thread(&sort_param->sort_info->info->rec_cache);
 
-  error=1;
+  /* Readers detach from the share if any. Avoid others to be blocked. */
+  if (sort_param->read_cache.share)
+    remove_io_thread(&sort_param->read_cache);
 
-  if (my_thread_init())
-    goto err;
+  mysql_mutex_lock(&sort_param->sort_info->mutex);
+  if (error)
+    sort_param->sort_info->got_error= 1;
 
-  { /* Add extra block since DBUG_ENTER declare variables */
-    DBUG_ENTER("_ma_thr_find_all_keys");
-    DBUG_PRINT("enter", ("master: %d", sort_param->master));
-    if (sort_param->sort_info->got_error)
-      goto err;
+  if (!--sort_param->sort_info->threads_running)
+    mysql_cond_signal(&sort_param->sort_info->cond);
+  mysql_mutex_unlock(&sort_param->sort_info->mutex);
 
-    if (sort_param->keyinfo->flag & HA_VAR_LENGTH_KEY)
-    {
-      sort_param->write_keys=     write_keys_varlen;
-      sort_param->read_to_buffer= read_to_buffer_varlen;
-      sort_param->write_key=      write_merge_key_varlen;
-    }
-    else
-    {
-      sort_param->write_keys=     write_keys;
-      sort_param->read_to_buffer= read_to_buffer;
-      sort_param->write_key=      write_merge_key;
-    }
-
-    my_b_clear(&sort_param->tempfile);
-    my_b_clear(&sort_param->tempfile_for_exceptions);
-    bzero((char*) &sort_param->buffpek,sizeof(sort_param->buffpek));
-    bzero((char*) &sort_param->unique, sizeof(sort_param->unique));
-
-    memavl=       max(sort_param->sortbuff_size, MIN_SORT_MEMORY);
-    idx=          (uint)sort_param->sort_info->max_records;
-    sort_length=  sort_param->key_length;
-    maxbuffer=    1;
-
-    while (memavl >= MIN_SORT_MEMORY)
-    {
-      if ((my_off_t) (idx+1)*(sort_length+sizeof(char*)) <= (my_off_t) memavl)
-        keys= idx+1;
-      else
-      {
-        ulong skr;
-        do
-        {
-          skr= maxbuffer;
-          if (memavl < sizeof(BUFFPEK)*maxbuffer ||
-              (keys=(memavl-sizeof(BUFFPEK)*maxbuffer)/
-               (sort_length+sizeof(char*))) <= 1 ||
-              keys < maxbuffer)
-          {
-            _ma_check_print_error(sort_param->sort_info->param,
-                                  "aria_sort_buffer_size is too small");
-            goto err;
-          }
-        }
-        while ((maxbuffer= (int) (idx/(keys-1)+1)) != skr);
-      }
-      if ((sort_keys= (uchar **)
-           my_malloc(keys*(sort_length+sizeof(char*))+
-                     ((sort_param->keyinfo->flag & HA_FULLTEXT) ?
-                      HA_FT_MAXBYTELEN : 0), MYF(0))))
-      {
-        if (my_init_dynamic_array(&sort_param->buffpek, sizeof(BUFFPEK),
-                                  maxbuffer, maxbuffer/2))
-        {
-          my_free(sort_keys);
-          sort_keys= (uchar **) NULL;            /* for err: label */
-        }
-        else
-          break;
-      }
-      old_memavl= memavl;
-      if ((memavl= memavl/4*3) < MIN_SORT_MEMORY &&
-          old_memavl > MIN_SORT_MEMORY)
-        memavl= MIN_SORT_MEMORY;
-    }
-    if (memavl < MIN_SORT_MEMORY)
-    {
-      _ma_check_print_error(sort_param->sort_info->param,
-                            "Aria sort buffer too small");
-      goto err; /* purecov: tested */
-    }
-
-    if (sort_param->sort_info->param->testflag & T_VERBOSE)
-      printf("Key %d - Allocating buffer for %lu keys\n",
-             sort_param->key+1, (ulong) keys);
-    sort_param->sort_keys= sort_keys;
-
-    idx= error= 0;
-    sort_keys[0]= (uchar*) (sort_keys+keys);
-
-    DBUG_PRINT("info", ("reading keys"));
-    while (!(error= sort_param->sort_info->got_error) &&
-           !(error= (*sort_param->key_read)(sort_param, sort_keys[idx])))
-    {
-      if (sort_param->real_key_length > sort_param->key_length)
-      {
-        if (write_key(sort_param,sort_keys[idx],
-                      &sort_param->tempfile_for_exceptions))
-          goto err;
-        continue;
-      }
-
-      if (++idx == keys)
-      {
-        if (sort_param->write_keys(sort_param, sort_keys, idx - 1,
-                                   (BUFFPEK *)alloc_dynamic(&sort_param->
-                                                            buffpek),
-                                   &sort_param->tempfile))
-          goto err;
-        sort_keys[0]= (uchar*) (sort_keys+keys);
-        memcpy(sort_keys[0], sort_keys[idx - 1],
-               (size_t) sort_param->key_length);
-        idx= 1;
-      }
-      sort_keys[idx]=sort_keys[idx - 1] + sort_param->key_length;
-    }
-    if (error > 0)
-      goto err;
-    if (sort_param->buffpek.elements)
-    {
-      if (sort_param->write_keys(sort_param,sort_keys, idx,
-                                 (BUFFPEK *) alloc_dynamic(&sort_param->
-                                                           buffpek),
-                                 &sort_param->tempfile))
-        goto err;
-      sort_param->keys= (sort_param->buffpek.elements - 1) * (keys - 1) + idx;
-    }
-    else
-      sort_param->keys= idx;
-
-    sort_param->sort_keys_length= keys;
-    goto ok;
-
-err:
-    DBUG_PRINT("error", ("got some error"));
-    sort_param->sort_info->got_error= 1; /* no need to protect with a mutex */
-    my_free(sort_keys);
-    sort_param->sort_keys=0;
-    delete_dynamic(& sort_param->buffpek);
-    close_cached_file(&sort_param->tempfile);
-    close_cached_file(&sort_param->tempfile_for_exceptions);
-
-ok:
-    free_root(&sort_param->wordroot, MYF(0));
-    /*
-      Detach from the share if the writer is involved. Avoid others to
-      be blocked. This includes a flush of the write buffer. This will
-      also indicate EOF to the readers.
-    */
-    if (sort_param->sort_info->info->rec_cache.share)
-      remove_io_thread(&sort_param->sort_info->info->rec_cache);
-
-    /* Readers detach from the share if any. Avoid others to be blocked. */
-    if (sort_param->read_cache.share)
-      remove_io_thread(&sort_param->read_cache);
-
-    mysql_mutex_lock(&sort_param->sort_info->mutex);
-    if (!--sort_param->sort_info->threads_running)
-      mysql_cond_signal(&sort_param->sort_info->cond);
-    mysql_mutex_unlock(&sort_param->sort_info->mutex);
-    DBUG_PRINT("exit", ("======== ending thread ========"));
-  }
   my_thread_end();
   return NULL;
 }
@@ -506,7 +518,7 @@ int _ma_thr_write_keys(MARIA_SORT_PARAM *sort_param)
 {
   MARIA_SORT_INFO *sort_info=sort_param->sort_info;
   HA_CHECK *param=sort_info->param;
-  ulong UNINIT_VAR(length), keys;
+  size_t UNINIT_VAR(length), keys;
   double *rec_per_key_part= param->new_rec_per_key_part;
   int got_error=sort_info->got_error;
   uint i;
@@ -563,18 +575,9 @@ int _ma_thr_write_keys(MARIA_SORT_PARAM *sort_param)
   {
     if (got_error)
       continue;
-    if (sinfo->keyinfo->flag & HA_VAR_LENGTH_KEY)
-    {
-      sinfo->write_keys=write_keys_varlen;
-      sinfo->read_to_buffer=read_to_buffer_varlen;
-      sinfo->write_key=write_merge_key_varlen;
-    }
-    else
-    {
-      sinfo->write_keys=write_keys;
-      sinfo->read_to_buffer=read_to_buffer;
-      sinfo->write_key=write_merge_key;
-    }
+
+    set_sort_param_read_write(sinfo);
+
     if (sinfo->buffpek.elements)
     {
       uint maxbuffer=sinfo->buffpek.elements-1;
