@@ -11,7 +11,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1335 USA */
 
 
 #include <my_global.h>
@@ -383,8 +383,8 @@ TRN *trnman_new_trn(WT_THD *wt)
     return 0;
   }
 
-  DBUG_PRINT("exit", ("trn: 0x%lx  trid: 0x%lu",
-                      (ulong) trn, (ulong) trn->trid));
+  DBUG_PRINT("exit", ("trn: %p  trid: 0x%lu  min_read_from: 0x%lu",
+                      trn, (ulong) trn->trid, (ulong) trn->min_read_from));
 
   DBUG_RETURN(trn);
 }
