@@ -1,4 +1,4 @@
-/* Copyright (c) 2000, 2011, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2000, 2013, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -11,7 +11,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1335  USA */
 
 	/* Functions to compressed records */
 
@@ -810,7 +810,7 @@ static void (*get_unpack_function(MI_COLUMNDEF *rec))
     return &uf_varchar2;
   case FIELD_LAST:
   default:
-    return 0;			/* This should never happend */
+    return 0;			/* This should never happen */
   }
 }
 
@@ -1052,7 +1052,7 @@ static void uf_blob(MI_COLUMNDEF *rec, MI_BIT_BUFF *bit_buff,
     }
     decode_bytes(rec,bit_buff,bit_buff->blob_pos,bit_buff->blob_pos+length);
     _mi_store_blob_length((uchar*) to,pack_length,length);
-    memcpy((char*) to+pack_length, &bit_buff->blob_pos, sizeof(char*));
+    memcpy(to+pack_length, &bit_buff->blob_pos, sizeof(char*));
     bit_buff->blob_pos+=length;
   }
 }
