@@ -12,7 +12,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1335  USA */
 
 /* key handling functions */
 
@@ -433,7 +433,7 @@ int _mi_prefix_search(MI_INFO *info, register MI_KEYDEF *keyinfo, uchar *page,
 
       /*
         If prefix_len > cmplen then we are in the end-space comparison
-        phase. Do not try to acces the key any more ==> left= 0.
+        phase. Do not try to access the key any more ==> left= 0.
       */
       left= ((len <= cmplen) ? suffix_len :
              ((prefix_len < cmplen) ? cmplen - prefix_len : 0));
@@ -608,11 +608,11 @@ void _mi_kpointer(register MI_INFO *info, register uchar *buff, my_off_t pos)
   case 5: mi_int5store(buff,pos); break;
 #else
   case 7: *buff++=0;
-    /* fall trough */
+    /* fall through */
   case 6: *buff++=0;
-    /* fall trough */
+    /* fall through */
   case 5: *buff++=0;
-    /* fall trough */
+    /* fall through */
 #endif
   case 4: mi_int4store(buff,pos); break;
   case 3: mi_int3store(buff,pos); break;
@@ -729,13 +729,13 @@ void _mi_dpointer(MI_INFO *info, uchar *buff, my_off_t pos)
   case 5: mi_int5store(buff,pos); break;
 #else
   case 8: *buff++=0;
-    /* fall trough */
+    /* fall through */
   case 7: *buff++=0;
-    /* fall trough */
+    /* fall through */
   case 6: *buff++=0;
-    /* fall trough */
+    /* fall through */
   case 5: *buff++=0;
-    /* fall trough */
+    /* fall through */
 #endif
   case 4: mi_int4store(buff,pos); break;
   case 3: mi_int3store(buff,pos); break;
