@@ -11,8 +11,8 @@ ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License along with
-this program; if not, write to the Free Software Foundation, Inc., 59 Temple
-Place, Suite 330, Boston, MA 02111-1307 USA
+this program; if not, write to the Free Software Foundation, Inc., 
+51 Franklin St, Fifth Floor, Boston, MA 02110-1335 USA
 
 *****************************************************************************/
 
@@ -33,6 +33,7 @@ Created 4/24/1996 Heikki Tuuri
 
 #include "btr0pcur.h"
 #include "btr0btr.h"
+#include "btr0sea.h"
 #include "page0page.h"
 #include "mach0data.h"
 #include "dict0dict.h"
@@ -104,7 +105,7 @@ dict_get_first_table_name_in_db(
 
 	mtr_start(&mtr);
 
-	sys_tables = dict_table_get_low("SYS_TABLES");
+	sys_tables = dict_table_get_low("SYS_TABLES", DICT_ERR_IGNORE_NONE);
 	sys_index = UT_LIST_GET_FIRST(sys_tables->indexes);
 	ut_a(!dict_table_is_comp(sys_tables));
 
@@ -270,7 +271,8 @@ dict_startscan_system(
 
 	ut_a(system_id < SYS_NUM_SYSTEM_TABLES);
 
-	system_table = dict_table_get_low(SYSTEM_TABLE_NAME[system_id]);
+	system_table = dict_table_get_low(SYSTEM_TABLE_NAME[system_id],
+					  DICT_ERR_IGNORE_NONE);
 
 	clust_index = UT_LIST_GET_FIRST(system_table->indexes);
 
@@ -335,7 +337,7 @@ dict_process_sys_tables_rec(
 	/* If DICT_TABLE_LOAD_FROM_CACHE is set, first check
 	whether there is cached dict_table_t struct first */
 	if (status & DICT_TABLE_LOAD_FROM_CACHE) {
-		*table = dict_table_get_low(table_name);
+		*table = dict_table_get_low(table_name, DICT_ERR_IGNORE_NONE);
 
 		if (!(*table)) {
 			err_msg = "Table not found in cache";
@@ -714,7 +716,7 @@ dict_sys_tables_get_flags(
 		return(ULINT_UNDEFINED);
 	}
 
-	if (UNIV_UNLIKELY(flags & (~0 << DICT_TF_BITS))) {
+	if (UNIV_UNLIKELY(flags & (~0U << DICT_TF_BITS))) {
 		/* Some unused bits are set. */
 		return(ULINT_UNDEFINED);
 	}
@@ -747,7 +749,7 @@ dict_check_tablespaces_and_store_max_id(
 
 	mtr_start(&mtr);
 
-	sys_tables = dict_table_get_low("SYS_TABLES");
+	sys_tables = dict_table_get_low("SYS_TABLES", DICT_ERR_IGNORE_NONE);
 	sys_index = UT_LIST_GET_FIRST(sys_tables->indexes);
 	ut_a(!dict_table_is_comp(sys_tables));
 
@@ -1030,7 +1032,7 @@ dict_load_columns(
 
 	mtr_start(&mtr);
 
-	sys_columns = dict_table_get_low("SYS_COLUMNS");
+	sys_columns = dict_table_get_low("SYS_COLUMNS", DICT_ERR_IGNORE_NONE);
 	sys_index = UT_LIST_GET_FIRST(sys_columns->indexes);
 	ut_a(!dict_table_is_comp(sys_columns));
 
@@ -1237,7 +1239,7 @@ dict_load_fields(
 
 	mtr_start(&mtr);
 
-	sys_fields = dict_table_get_low("SYS_FIELDS");
+	sys_fields = dict_table_get_low("SYS_FIELDS", DICT_ERR_IGNORE_NONE);
 	sys_index = UT_LIST_GET_FIRST(sys_fields->indexes);
 	ut_a(!dict_table_is_comp(sys_fields));
 	ut_a(name_of_col_is(sys_fields, sys_index, 4, "COL_NAME"));
@@ -1403,7 +1405,7 @@ err_len:
 		goto err_len;
 	}
 	type = mach_read_from_4(field);
-	if (UNIV_UNLIKELY(type & (~0 << DICT_IT_BITS))) {
+	if (UNIV_UNLIKELY(type & (~0U << DICT_IT_BITS))) {
 		return("unknown SYS_INDEXES.TYPE bits");
 	}
 
@@ -1430,6 +1432,7 @@ err_len:
 
 	(*index)->id = id;
 	(*index)->page = mach_read_from_4(field);
+	btr_search_index_init(*index);
 	ut_ad((*index)->page);
 
 	return(NULL);
@@ -1464,7 +1467,7 @@ dict_load_indexes(
 
 	mtr_start(&mtr);
 
-	sys_indexes = dict_table_get_low("SYS_INDEXES");
+	sys_indexes = dict_table_get_low("SYS_INDEXES", DICT_ERR_IGNORE_NONE);
 	sys_index = UT_LIST_GET_FIRST(sys_indexes->indexes);
 	ut_a(!dict_table_is_comp(sys_indexes));
 	ut_a(name_of_col_is(sys_indexes, sys_index, 4, "NAME"));
@@ -1767,7 +1770,7 @@ err_len:
 
 		flags2 = mach_read_from_4(field);
 
-		if (flags2 & (~0 << (DICT_TF2_BITS - DICT_TF2_SHIFT))) {
+		if (flags2 & (~0U << (DICT_TF2_BITS - DICT_TF2_SHIFT))) {
 			ut_print_timestamp(stderr);
 			fputs("  InnoDB: Warning: table ", stderr);
 			ut_print_filename(stderr, name);
@@ -1776,7 +1779,7 @@ err_len:
 				" has unknown flags %lx.\n",
 				(ulong) flags2);
 
-			flags2 &= ~(~0 << (DICT_TF2_BITS - DICT_TF2_SHIFT));
+			flags2 &= ~(~0U << (DICT_TF2_BITS - DICT_TF2_SHIFT));
 		}
 
 		flags |= flags2 << DICT_TF2_SHIFT;
@@ -1830,13 +1833,15 @@ dict_load_table(
 	const char*	err_msg;
 	mtr_t		mtr;
 
+	DBUG_ENTER("dict_load_table");
+
 	ut_ad(mutex_own(&(dict_sys->mutex)));
 
 	heap = mem_heap_create(32000);
 
 	mtr_start(&mtr);
 
-	sys_tables = dict_table_get_low("SYS_TABLES");
+	sys_tables = dict_table_get_low("SYS_TABLES", DICT_ERR_IGNORE_NONE);
 	sys_index = UT_LIST_GET_FIRST(sys_tables->indexes);
 	ut_a(!dict_table_is_comp(sys_tables));
 	ut_a(name_of_col_is(sys_tables, sys_index, 3, "ID"));
@@ -1863,7 +1868,7 @@ err_exit:
 		mtr_commit(&mtr);
 		mem_heap_free(heap);
 
-		return(NULL);
+		DBUG_RETURN(NULL);
 	}
 
 	field = rec_get_nth_field_old(rec, 0, &len);
@@ -1907,7 +1912,7 @@ err_exit:
 			if (!fil_open_single_table_tablespace(
 				TRUE, table->space,
 				table->flags == DICT_TF_COMPACT ? 0 :
-				table->flags & ~(~0 << DICT_TF_BITS), name, NULL)) {
+				table->flags & ~(~0U << DICT_TF_BITS), name, NULL)) {
 				/* We failed to find a sensible
 				tablespace file */
 
@@ -1965,9 +1970,16 @@ err_exit:
 	all indexes were loaded. */
 	if (!cached) {
 	} else if (err == DB_SUCCESS) {
-		err = dict_load_foreigns(table->name, TRUE, TRUE);
+		err = dict_load_foreigns(table->name, TRUE, TRUE,
+					 ignore_err);
 
 		if (err != DB_SUCCESS) {
+			fprintf(stderr,
+				"InnoDB: Load table '%s' failed, the table "
+				"has missing foreign key indexes. Turn off "
+				"'foreign_key_checks' and try again.",
+				table->name);
+
 			dict_table_remove_from_cache(table);
 			table = NULL;
 		} else {
@@ -2009,7 +2021,7 @@ err_exit:
 			"InnoDB: and recreate the foreign key table"
 			" or the referenced table.\n"
 			"InnoDB: Submit a detailed bug report"
-			" to http://bugs.mysql.com\n"
+			" to https://jira.mariadb.org/\n"
 			"InnoDB: Latest foreign key error printout:\n%s\n",
 			dict_foreign_err_buf);
 
@@ -2018,8 +2030,8 @@ err_exit:
 #endif /* 0 */
 func_exit:
 	mem_heap_free(heap);
-
-	return(table);
+	ut_ad(table == NULL || dict_table_check_foreign_keys(table));
+	DBUG_RETURN(table);
 }
 
 /***********************************************************************//**
@@ -2165,7 +2177,8 @@ dict_load_foreign_cols(
 		foreign->heap, foreign->n_fields * sizeof(void*));
 	mtr_start(&mtr);
 
-	sys_foreign_cols = dict_table_get_low("SYS_FOREIGN_COLS");
+	sys_foreign_cols = dict_table_get_low("SYS_FOREIGN_COLS",
+					      DICT_ERR_IGNORE_NONE);
 	sys_index = UT_LIST_GET_FIRST(sys_foreign_cols->indexes);
 	ut_a(!dict_table_is_comp(sys_foreign_cols));
 
@@ -2185,6 +2198,23 @@ dict_load_foreign_cols(
 		ut_a(!rec_get_deleted_flag(rec, 0));
 
 		field = rec_get_nth_field_old(rec, 0, &len);
+
+if (len != id_len || ut_memcmp(id, field, len) != 0) {
+			char * tmp,*tmp2;
+
+			tmp = mem_heap_alloc(foreign->heap, id_len+4);
+			ut_memcpy(tmp, id, id_len);
+			tmp[id_len]='\0';
+			tmp2 = mem_heap_alloc(foreign->heap, len+4);
+			ut_memcpy(tmp2, id, len);
+			tmp2[len]='\0';
+			fprintf(stderr, "InnoDB: Error: len = %lu != id_len %lu\n",
+				len, id_len);
+			fprintf(stderr, "InnoDB: Error: id %s != field %s\n",
+				tmp, tmp2);
+
+		}
+
 		ut_a(len == id_len);
 		ut_a(ut_memcmp(id, field, len) == 0);
 
@@ -2214,15 +2244,19 @@ static
 ulint
 dict_load_foreign(
 /*==============*/
-	const char*	id,	/*!< in: foreign constraint id, not
+	const char*		id,
+				/*!< in: foreign constraint id, not
 				necessary '\0'-terminated */
-	ulint		id_len,	/*!< in: id length */
-	ibool		check_charsets,
+	ulint			id_len,
+				/*!< in: id length */
+	ibool			check_charsets,
 				/*!< in: TRUE=check charset compatibility */
-	ibool		check_recursive)
+	ibool			check_recursive,
 				/*!< in: Whether to record the foreign table
 				parent count to avoid unlimited recursive
 				load of chained foreign tables */
+	dict_err_ignore_t	ignore_err)
+				/*!< in: error to be ignored */
 {
 	dict_foreign_t*	foreign;
 	dict_table_t*	sys_foreign;
@@ -2239,13 +2273,15 @@ dict_load_foreign(
 	dict_table_t*	for_table;
 	dict_table_t*	ref_table;
 
+	DBUG_ENTER("dict_load_foreign");
+
 	ut_ad(mutex_own(&(dict_sys->mutex)));
 
 	heap2 = mem_heap_create(1000);
 
 	mtr_start(&mtr);
 
-	sys_foreign = dict_table_get_low("SYS_FOREIGN");
+	sys_foreign = dict_table_get_low("SYS_FOREIGN", DICT_ERR_IGNORE_NONE);
 	sys_index = UT_LIST_GET_FIRST(sys_foreign->indexes);
 	ut_a(!dict_table_is_comp(sys_foreign));
 
@@ -2271,7 +2307,7 @@ dict_load_foreign(
 		mtr_commit(&mtr);
 		mem_heap_free(heap2);
 
-		return(DB_ERROR);
+		DBUG_RETURN(DB_ERROR);
 	}
 
 	field = rec_get_nth_field_old(rec, 0, &len);
@@ -2287,7 +2323,7 @@ dict_load_foreign(
 		mtr_commit(&mtr);
 		mem_heap_free(heap2);
 
-		return(DB_ERROR);
+		DBUG_RETURN(DB_ERROR);
 	}
 
 	/* Read the table names and the number of columns associated
@@ -2359,7 +2395,9 @@ dict_load_foreign(
 		have to load it so that we are able to make type comparisons
 		in the next function call. */
 
-		for_table = dict_table_get_low(foreign->foreign_table_name_lookup);
+		for_table = dict_table_get_low(
+				foreign->foreign_table_name_lookup,
+				DICT_ERR_IGNORE_NONE);
 
 		if (for_table && ref_table && check_recursive) {
 			/* This is to record the longest chain of ancesters
@@ -2382,7 +2420,7 @@ dict_load_foreign(
 	a new foreign key constraint but loading one from the data
 	dictionary. */
 
-	return(dict_foreign_add_to_cache(foreign, check_charsets));
+	DBUG_RETURN(dict_foreign_add_to_cache(foreign, check_charsets, ignore_err));
 }
 
 /***********************************************************************//**
@@ -2396,11 +2434,13 @@ UNIV_INTERN
 ulint
 dict_load_foreigns(
 /*===============*/
-	const char*	table_name,	/*!< in: table name */
-	ibool		check_recursive,/*!< in: Whether to check recursive
-					load of tables chained by FK */
-	ibool		check_charsets)	/*!< in: TRUE=check charset
-					compatibility */
+	const char*		table_name,	/*!< in: table name */
+	ibool			check_recursive,/*!< in: Whether to check
+						recursive load of tables
+						chained by FK */
+	ibool			check_charsets,	/*!< in: TRUE=check charset
+						compatibility */
+	dict_err_ignore_t	ignore_err)	/*!< in: error to be ignored */
 {
 	ulint		tuple_buf[(DTUPLE_EST_ALLOC(1) + sizeof(ulint) - 1)
 				/ sizeof(ulint)];
@@ -2415,9 +2455,11 @@ dict_load_foreigns(
 	ulint		err;
 	mtr_t		mtr;
 
+	DBUG_ENTER("dict_load_foreigns");
+
 	ut_ad(mutex_own(&(dict_sys->mutex)));
 
-	sys_foreign = dict_table_get_low("SYS_FOREIGN");
+	sys_foreign = dict_table_get_low("SYS_FOREIGN", DICT_ERR_IGNORE_NONE);
 
 	if (sys_foreign == NULL) {
 		/* No foreign keys defined yet in this database */
@@ -2426,7 +2468,7 @@ dict_load_foreigns(
 			"InnoDB: Error: no foreign key system tables"
 			" in the database\n");
 
-		return(DB_ERROR);
+		DBUG_RETURN(DB_ERROR);
 	}
 
 	ut_a(!dict_table_is_comp(sys_foreign));
@@ -2501,12 +2543,12 @@ loop:
 	/* Load the foreign constraint definition to the dictionary cache */
 
 	err = dict_load_foreign((char*) field, len, check_charsets,
-				check_recursive);
+				check_recursive, ignore_err);
 
 	if (err != DB_SUCCESS) {
 		btr_pcur_close(&pcur);
 
-		return(err);
+		DBUG_RETURN(err);
 	}
 
 	mtr_start(&mtr);
@@ -2535,5 +2577,75 @@ load_next_index:
 		goto start_load;
 	}
 
-	return(DB_SUCCESS);
+	DBUG_RETURN(DB_SUCCESS);
+}
+
+/********************************************************************//**
+Check if dict_table_t::foreign_rbt and dict_table::foreign_list
+contain the same set of foreign key objects; and check if
+dict_table_t::referenced_rbt and dict_table::referenced_list contain
+the same set of foreign key objects.
+@return	TRUE if correct, FALSE otherwise. */
+UNIV_INTERN
+ibool
+dict_table_check_foreign_keys(
+/*==========================*/
+	const dict_table_t* table)	/* in: table object to check */
+{
+	dict_foreign_t*		foreign;
+	const ib_rbt_node_t*	node;
+
+	ut_ad(mutex_own(&(dict_sys->mutex)));
+
+	if (table->foreign_rbt == NULL) {
+
+		if (UT_LIST_GET_LEN(table->foreign_list) > 0) {
+			return(FALSE);
+		}
+
+	} else {
+
+		if (UT_LIST_GET_LEN(table->foreign_list)
+		    != rbt_size(table->foreign_rbt)) {
+			return(FALSE);
+		}
+
+		foreign = UT_LIST_GET_FIRST(table->foreign_list);
+
+		while (foreign != NULL) {
+
+			node = rbt_lookup(table->foreign_rbt, foreign->id);
+			if (node == NULL) {
+				return(FALSE);
+			}
+			foreign = UT_LIST_GET_NEXT(foreign_list, foreign);
+		}
+	}
+
+	if (table->referenced_rbt == NULL ) {
+
+		if (UT_LIST_GET_LEN(table->referenced_list) > 0) {
+			return(FALSE);
+		}
+
+	} else {
+
+		if (UT_LIST_GET_LEN(table->referenced_list)
+		    != rbt_size(table->referenced_rbt)) {
+			return(FALSE);
+		}
+
+		foreign = UT_LIST_GET_FIRST(table->referenced_list);
+
+		while (foreign != NULL) {
+
+			node = rbt_lookup(table->referenced_rbt, foreign->id);
+			if (node == NULL) {
+				return(FALSE);
+			}
+			foreign = UT_LIST_GET_NEXT(referenced_list, foreign);
+		}
+	}
+
+	return(TRUE);
 }
