@@ -1,6 +1,7 @@
 /*****************************************************************************
 
 Copyright (c) 1996, 2010, Innobase Oy. All Rights Reserved.
+Copyright (c) 2019, MariaDB Corporation.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free Software
@@ -11,8 +12,8 @@ ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License along with
-this program; if not, write to the Free Software Foundation, Inc., 59 Temple
-Place, Suite 330, Boston, MA 02111-1307 USA
+this program; if not, write to the Free Software Foundation, Inc., 
+51 Franklin St, Fifth Floor, Boston, MA 02110-1335 USA
 
 *****************************************************************************/
 
@@ -49,12 +50,14 @@ the kernel mutex */
 extern ulint	trx_n_prepared;
 
 /********************************************************************//**
-Releases the search latch if trx has reserved it. */
-UNIV_INTERN
+In XtraDB it is impossible for a transaction to own a search latch outside of
+InnoDB code, so there is nothing to release on demand.  We keep this function to
+simplify maintenance.*/
+UNIV_INLINE
 void
 trx_search_latch_release_if_reserved(
 /*=================================*/
-	trx_t*	   trx); /*!< in: transaction */
+	trx_t*	   trx __attribute__((unused))); /*!< in: transaction */
 /******************************************************************//**
 Set detailed error message for the transaction. */
 UNIV_INTERN
@@ -88,7 +91,7 @@ trx_t*
 trx_create(
 /*=======*/
 	sess_t*	sess)	/*!< in: session */
-	__attribute__((nonnull));
+	__attribute__((warn_unused_result));
 /********************************************************************//**
 Creates a transaction object for MySQL.
 @return	own: transaction object */
@@ -117,7 +120,7 @@ void
 trx_free_prepared(
 /*==============*/
 	trx_t*	trx)	/*!< in, own: trx object */
-	UNIV_COLD __attribute__((nonnull));
+	UNIV_COLD;
 /********************************************************************//**
 Frees a transaction object for MySQL. */
 UNIV_INTERN
@@ -506,6 +509,7 @@ struct trx_struct{
 					trx_release_descriptor(). Different name
 					ensures we notice any new code changing
 					the state. */
+
 	/*------------------------------*/
 	/* MySQL has a transaction coordinator to coordinate two phase
        	commit between multiple storage engines and the binary log. When
@@ -555,8 +559,8 @@ struct trx_struct{
 					in that case we must flush the log
 					in trx_commit_complete_for_mysql() */
 	ulint		duplicates;	/*!< TRX_DUP_IGNORE | TRX_DUP_REPLACE */
-	ulint		has_search_latch;
-					/* TRUE if this trx has latched the
+	ibool		has_search_latch;
+					/* TRUE if this trx has latched any
 					search system latch in S-mode */
 	ulint		deadlock_mark;	/*!< a mark field used in deadlock
 					checking algorithm.  */
@@ -799,6 +803,8 @@ struct trx_struct{
 #define	TRX_ACTIVE		1
 #define	TRX_COMMITTED_IN_MEMORY	2
 #define	TRX_PREPARED		3	/* Support for 2PC/XA */
+#define TRX_PREPARED_RECOVERED	4	/* XA PREPARE transaction that
+					was returned to ha_recover() */
 
 /* Transaction execution states when trx->conc_state == TRX_ACTIVE */
 #define TRX_QUE_RUNNING		0	/* transaction is running */
