@@ -11,8 +11,8 @@ ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License along with
-this program; if not, write to the Free Software Foundation, Inc., 59 Temple
-Place, Suite 330, Boston, MA 02111-1307 USA
+this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
+Street, Fifth Floor, Boston, MA 02110-1335  USA
 
 *****************************************************************************/
 
@@ -34,6 +34,14 @@ Online database log parsing for changed page tracking
 #include "trx0sys.h"
 #include "ut0rbt.h"
 
+#ifdef __WIN__
+/* error LNK2001: unresolved external symbol _debug_sync_C_callback_ptr */
+# define DEBUG_SYNC_C(dummy) ((void) 0)
+#else
+# include "m_string.h" /* for my_sys.h */
+# include "my_sys.h" /* DEBUG_SYNC_C */
+#endif
+
 enum { FOLLOW_SCAN_SIZE = 4 * (UNIV_PAGE_SIZE_MAX) };
 
 #ifdef UNIV_PFS_MUTEX
@@ -76,12 +84,14 @@ struct log_bitmap_struct {
 					both the correct type and the tree does
 					not mind its overwrite during
 					rbt_next() tree traversal. */
-	mutex_t		mutex;		/*!< mutex protecting all the fields.*/
 };
 
 /* The log parsing and bitmap output struct instance */
 static struct log_bitmap_struct* log_bmp_sys;
 
+/* Mutex protecting log_bmp_sys */
+static mutex_t	log_bmp_sys_mutex;
+
 /** File name stem for bitmap files. */
 static const char* bmp_file_name_stem = "ib_modified_log_";
 
@@ -180,7 +190,7 @@ log_online_set_page_bit(
 	byte		search_page[MODIFIED_PAGE_BLOCK_SIZE];
 	byte		*page_ptr;
 
-	ut_ad(mutex_own(&log_bmp_sys->mutex));
+	ut_ad(mutex_own(&log_bmp_sys_mutex));
 
 	ut_a(space != ULINT_UNDEFINED);
 	ut_a(page_no != ULINT_UNDEFINED);
@@ -331,7 +341,7 @@ log_online_read_last_tracked_lsn(void)
 	ib_uint64_t	result;
 	ib_uint64_t	read_offset	= log_bmp_sys->out.offset;
 
-	while (!checksum_ok && read_offset > 0 && !is_last_page)
+	while ((!checksum_ok || !is_last_page) && read_offset > 0)
 	{
 		read_offset -= MODIFIED_PAGE_BLOCK_SIZE;
 		log_bmp_sys->out.offset = read_offset;
@@ -491,9 +501,9 @@ log_online_make_bitmap_name(
 /*=========================*/
 	ib_uint64_t	start_lsn)	/*!< in: the start LSN name part */
 {
-	ut_snprintf(log_bmp_sys->out.name, FN_REFLEN, bmp_file_name_template,
-		    log_bmp_sys->bmp_file_home, bmp_file_name_stem,
-		    log_bmp_sys->out_seq_num, start_lsn);
+	ut_snprintf(log_bmp_sys->out.name, sizeof(log_bmp_sys->out.name), 
+            bmp_file_name_template, log_bmp_sys->bmp_file_home,
+            bmp_file_name_stem, log_bmp_sys->out_seq_num, start_lsn);
 }
 
 /*********************************************************************//**
@@ -536,7 +546,7 @@ log_online_start_bitmap_file(void)
 							innodb_file_bmp_key,
 							log_bmp_sys->out.name,
 							OS_FILE_CREATE,
-							OS_FILE_READ_WRITE,
+							OS_FILE_READ_WRITE_CACHED,
 							&success);
 	}
 	if (UNIV_UNLIKELY(!success)) {
@@ -601,12 +611,19 @@ log_online_is_bitmap_file(
 		&& (!strcmp(stem, bmp_file_name_stem)));
 }
 
-/*********************************************************************//**
-Initialize the online log following subsytem. */
+/** Initialize the constant part of the log tracking subsystem */
+UNIV_INTERN
+void
+log_online_init(void)
+{
+	mutex_create(log_bmp_sys_mutex_key, &log_bmp_sys_mutex,
+		     SYNC_LOG_ONLINE);
+}
+
+/** Initialize the dynamic part of the log tracking subsystem */
 UNIV_INTERN
 void
 log_online_read_init(void)
-/*======================*/
 {
 	ibool		success;
 	ib_uint64_t	tracking_start_lsn
@@ -627,9 +644,6 @@ log_online_read_init(void)
 	log_bmp_sys->read_buf = ut_align(log_bmp_sys->read_buf_ptr,
 					 OS_FILE_LOG_BLOCK_SIZE);
 
-	mutex_create(log_bmp_sys_mutex_key, &log_bmp_sys->mutex,
-		     SYNC_LOG_ONLINE);
-
 	/* Initialize bitmap file directory from srv_data_home and add a path
 	separator if needed.  */
 	srv_data_home_len = strlen(srv_data_home);
@@ -696,7 +710,7 @@ log_online_read_init(void)
 	log_bmp_sys->out.file
 		= os_file_create_simple_no_error_handling
 		(innodb_file_bmp_key, log_bmp_sys->out.name, OS_FILE_OPEN,
-		 OS_FILE_READ_WRITE, &success);
+		 OS_FILE_READ_WRITE_CACHED, &success);
 
 	if (!success) {
 
@@ -712,6 +726,7 @@ log_online_read_init(void)
 		ulint		size_high;
 		ib_uint64_t	last_tracked_lsn;
 		ib_uint64_t	file_start_lsn;
+		ibool		need_rotate;
 
 		success = os_file_get_size(log_bmp_sys->out.file, &size_low,
 					   &size_high);
@@ -734,7 +749,11 @@ log_online_read_init(void)
 		}
 
 		last_tracked_lsn = log_online_read_last_tracked_lsn();
+		/* Do not rotate if we truncated the file to zero length - we
+		can just start writing there */
+		need_rotate = (last_tracked_lsn != 0);
 		if (!last_tracked_lsn) {
+
 			last_tracked_lsn = last_file_start_lsn;
 		}
 
@@ -746,7 +765,10 @@ log_online_read_init(void)
 		} else {
 			file_start_lsn = tracking_start_lsn;
 		}
-		if (!log_online_rotate_bitmap_file(file_start_lsn)) {
+
+		if (need_rotate
+		    && !log_online_rotate_bitmap_file(file_start_lsn)) {
+
 			exit(1);
 		}
 
@@ -774,14 +796,18 @@ log_online_read_init(void)
 	log_set_tracked_lsn(tracking_start_lsn);
 }
 
-/*********************************************************************//**
-Shut down the online log following subsystem. */
+/** Shut down the dynamic part of the log tracking subsystem */
 UNIV_INTERN
 void
 log_online_read_shutdown(void)
-/*==========================*/
 {
-	ib_rbt_node_t *free_list_node = log_bmp_sys->page_free_list;
+	ib_rbt_node_t *free_list_node;
+
+	mutex_enter(&log_bmp_sys_mutex);
+
+	srv_track_changed_pages = FALSE;
+
+	free_list_node = log_bmp_sys->page_free_list;
 
 	if (log_bmp_sys->out.file != os_file_invalid) {
 		os_file_close(log_bmp_sys->out.file);
@@ -796,10 +822,21 @@ log_online_read_shutdown(void)
 		free_list_node = next;
 	}
 
-	mutex_free(&log_bmp_sys->mutex);
-
 	ut_free(log_bmp_sys->read_buf_ptr);
 	ut_free(log_bmp_sys);
+	log_bmp_sys = NULL;
+
+	srv_redo_log_thread_started = FALSE;
+
+	mutex_exit(&log_bmp_sys_mutex);
+}
+
+/** Shut down the constant part of the log tracking subsystem */
+UNIV_INTERN
+void
+log_online_shutdown(void)
+{
+	mutex_free(&log_bmp_sys_mutex);
 }
 
 /*********************************************************************//**
@@ -850,7 +887,7 @@ log_online_parse_redo_log(void)
 
 	ulint len = 0;
 
-	ut_ad(mutex_own(&log_bmp_sys->mutex));
+	ut_ad(mutex_own(&log_bmp_sys_mutex));
 
 	while (ptr != end
 	       && log_bmp_sys->next_parse_lsn < log_bmp_sys->end_lsn) {
@@ -942,7 +979,7 @@ log_online_add_to_parse_buf(
 	ulint actual_data_len = (end_offset >= start_offset)
 		? end_offset - start_offset : 0;
 
-	ut_ad(mutex_own(&log_bmp_sys->mutex));
+	ut_ad(mutex_own(&log_bmp_sys_mutex));
 
 	ut_memcpy(log_bmp_sys->parse_buf_end, log_block + start_offset,
 		  actual_data_len);
@@ -968,7 +1005,7 @@ log_online_parse_redo_log_block(
 {
 	ulint block_data_len;
 
-	ut_ad(mutex_own(&log_bmp_sys->mutex));
+	ut_ad(mutex_own(&log_bmp_sys_mutex));
 
 	block_data_len = log_block_get_data_len(log_block);
 
@@ -996,7 +1033,7 @@ log_online_follow_log_seg(
 	byte* log_block_end = log_bmp_sys->read_buf
 		+ (block_end_lsn - block_start_lsn);
 
-	ut_ad(mutex_own(&log_bmp_sys->mutex));
+	ut_ad(mutex_own(&log_bmp_sys_mutex));
 
 	mutex_enter(&log_sys->mutex);
 	log_group_read_log_seg(LOG_RECOVER, log_bmp_sys->read_buf,
@@ -1060,7 +1097,7 @@ log_online_follow_log_group(
 	ib_uint64_t block_start_lsn = contiguous_lsn;
 	ib_uint64_t block_end_lsn;
 
-	ut_ad(mutex_own(&log_bmp_sys->mutex));
+	ut_ad(mutex_own(&log_bmp_sys_mutex));
 
 	log_bmp_sys->next_parse_lsn = log_bmp_sys->start_lsn;
 	log_bmp_sys->parse_buf_end = log_bmp_sys->parse_buf;
@@ -1100,10 +1137,35 @@ log_online_write_bitmap_page(
 {
 	ibool	success;
 
-	ut_ad(mutex_own(&log_bmp_sys->mutex));
+	ut_ad(mutex_own(&log_bmp_sys_mutex));
 
 	/* Simulate a write error */
-	DBUG_EXECUTE_IF("bitmap_page_write_error", return FALSE;);
+	DBUG_EXECUTE_IF("bitmap_page_write_error",
+			{
+				ulint space_id
+					= mach_read_from_4(block
+					+ MODIFIED_PAGE_SPACE_ID);
+				if (space_id > 0) {
+					fprintf(stderr, "InnoDB: Error: "
+						"simulating bitmap write "
+						"error in "
+						"log_online_write_bitmap_page "
+						"for space ID %lu\n",
+						space_id);
+					return FALSE;
+				}
+			});
+
+	/* A crash injection site that ensures last checkpoint LSN > last
+	tracked LSN, so that LSN tracking for this interval is tested. */
+	DBUG_EXECUTE_IF("crash_before_bitmap_write",
+			{
+				ulint space_id
+					= mach_read_from_4(block
+					+ MODIFIED_PAGE_SPACE_ID);
+				if (space_id > 0)
+					DBUG_SUICIDE();
+			});
 
 	success = os_file_write(log_bmp_sys->out.name, log_bmp_sys->out.file,
 				block,
@@ -1153,7 +1215,7 @@ log_online_write_bitmap(void)
 	const ib_rbt_node_t	*last_bmp_tree_node;
 	ibool			success = TRUE;
 
-	ut_ad(mutex_own(&log_bmp_sys->mutex));
+	ut_ad(mutex_own(&log_bmp_sys_mutex));
 
 	if (log_bmp_sys->out.offset >= srv_max_bitmap_file_size) {
 		if (!log_online_rotate_bitmap_file(log_bmp_sys->start_lsn)) {
@@ -1194,6 +1256,13 @@ log_online_write_bitmap(void)
 
 		bmp_tree_node = (ib_rbt_node_t*)
 			rbt_next(log_bmp_sys->modified_pages, bmp_tree_node);
+
+		DBUG_EXECUTE_IF("bitmap_page_2_write_error",
+				if (bmp_tree_node)
+				{
+					DBUG_SET("+d,bitmap_page_write_error");
+					DBUG_SET("-d,bitmap_page_2_write_error");
+				});
 	}
 
 	rbt_reset(log_bmp_sys->modified_pages);
@@ -1214,11 +1283,16 @@ log_online_follow_redo_log(void)
 	log_group_t*	group;
 	ibool		result;
 
-	mutex_enter(&log_bmp_sys->mutex);
+	if (!srv_track_changed_pages)
+		return TRUE;
+
+	DEBUG_SYNC_C("log_online_follow_redo_log");
+
+	mutex_enter(&log_bmp_sys_mutex);
 
 	if (!srv_track_changed_pages) {
-		mutex_exit(&log_bmp_sys->mutex);
-		return FALSE;
+		mutex_exit(&log_bmp_sys_mutex);
+		return TRUE;
 	}
 
 	/* Grab the LSN of the last checkpoint, we will parse up to it */
@@ -1227,7 +1301,7 @@ log_online_follow_redo_log(void)
 	mutex_exit(&(log_sys->mutex));
 
 	if (log_bmp_sys->end_lsn == log_bmp_sys->start_lsn) {
-		mutex_exit(&log_bmp_sys->mutex);
+		mutex_exit(&log_bmp_sys_mutex);
 		return TRUE;
 	}
 
@@ -1242,18 +1316,33 @@ log_online_follow_redo_log(void)
 		group = UT_LIST_GET_NEXT(log_groups, group);
 	}
 
-	/* A crash injection site that ensures last checkpoint LSN > last
-	tracked LSN, so that LSN tracking for this interval is tested. */
-	DBUG_EXECUTE_IF("crash_before_bitmap_write", DBUG_SUICIDE(););
-
 	result = log_online_write_bitmap();
 	log_bmp_sys->start_lsn = log_bmp_sys->end_lsn;
 	log_set_tracked_lsn(log_bmp_sys->start_lsn);
 
-	mutex_exit(&log_bmp_sys->mutex);
+	mutex_exit(&log_bmp_sys_mutex);
 	return result;
 }
 
+/*********************************************************************//**
+Diagnose a bitmap file range setup failure and free the partially-initialized
+bitmap file range.  */
+UNIV_COLD
+static
+void
+log_online_diagnose_inconsistent_dir(
+/*=================================*/
+	log_online_bitmap_file_range_t	*bitmap_files)	/*!<in/out: bitmap file
+							range */
+{
+	fprintf(stderr,
+		"InnoDB: Warning: inconsistent bitmap file "
+		"directory for a "
+		"INFORMATION_SCHEMA.INNODB_CHANGED_PAGES query"
+		"\n");
+	free(bitmap_files->files);
+}
+
 /*********************************************************************//**
 List the bitmap files in srv_data_home and setup their range that contains the
 specified LSN interval.  This range, if non-empty, will start with a file that
@@ -1355,6 +1444,8 @@ log_online_setup_bitmap_file_range(
 
 	bitmap_files->count = last_file_seq_num - first_file_seq_num + 1;
 
+	DEBUG_SYNC_C("setup_bitmap_range_middle");
+
 	/* 2nd pass: get the file names in the file_seq_num order */
 
 	bitmap_dir = os_file_opendir(srv_data_home, FALSE);
@@ -1390,12 +1481,7 @@ log_online_setup_bitmap_file_range(
 		array_pos = file_seq_num - first_file_seq_num;
 		if (UNIV_UNLIKELY(array_pos >= bitmap_files->count)) {
 
-			fprintf(stderr,
-				"InnoDB: Error: inconsistent bitmap file "
-				"directory for a "
-				"INFORMATION_SCHEMA.INNODB_CHANGED_PAGES query"
-				"\n");
-			free(bitmap_files->files);
+			log_online_diagnose_inconsistent_dir(bitmap_files);
 			return FALSE;
 		}
 
@@ -1421,22 +1507,30 @@ log_online_setup_bitmap_file_range(
 		return FALSE;
 	}
 
-#ifdef UNIV_DEBUG
-	ut_ad(bitmap_files->files[0].seq_num == first_file_seq_num);
-	ut_ad(bitmap_files->files[0].start_lsn == first_file_start_lsn);
+	if (!bitmap_files->files[0].seq_num
+	    || bitmap_files->files[0].seq_num != first_file_seq_num) {
+
+		log_online_diagnose_inconsistent_dir(bitmap_files);
+		return FALSE;
+	}
+
 	{
 		size_t i;
 		for (i = 1; i < bitmap_files->count; i++) {
 			if (!bitmap_files->files[i].seq_num) {
 				break;
 			}
-			ut_ad(bitmap_files->files[i].seq_num
-			      > bitmap_files->files[i - 1].seq_num);
-			ut_ad(bitmap_files->files[i].start_lsn
-			      >= bitmap_files->files[i - 1].start_lsn);
+			if ((bitmap_files->files[i].seq_num
+			      <= bitmap_files->files[i - 1].seq_num)
+			    || (bitmap_files->files[i].start_lsn
+				< bitmap_files->files[i - 1].start_lsn)) {
+
+				log_online_diagnose_inconsistent_dir(
+								bitmap_files);
+				return FALSE;
+			}
 		}
 	}
-#endif
 
 	return TRUE;
 }
@@ -1459,10 +1553,20 @@ log_online_open_bitmap_file_read_only(
 	ibool	success	= FALSE;
 	ulint	size_low;
 	ulint	size_high;
+	size_t  srv_data_home_len;
 
 	ut_ad(name[0] != '\0');
 
-	ut_snprintf(bitmap_file->name, FN_REFLEN, "%s%s", srv_data_home, name);
+	srv_data_home_len = strlen(srv_data_home);
+	if (srv_data_home_len
+			&& srv_data_home[srv_data_home_len-1]
+			!= SRV_PATH_SEPARATOR) {
+		ut_snprintf(bitmap_file->name, sizeof(bitmap_file->name), "%s%c%s",
+				srv_data_home, SRV_PATH_SEPARATOR, name);
+	} else {
+		ut_snprintf(bitmap_file->name, sizeof(bitmap_file->name), "%s%s",
+				srv_data_home, name);
+	}
 	bitmap_file->file
 		= os_file_create_simple_no_error_handling(innodb_file_bmp_key,
 							  bitmap_file->name,
@@ -1563,6 +1667,19 @@ log_online_bitmap_iterator_init(
 {
 	ut_a(i);
 
+	i->max_lsn = max_lsn;
+
+	if (UNIV_UNLIKELY(min_lsn > max_lsn)) {
+
+		/* Empty range */
+		i->in_files.count = 0;
+		i->in_files.files = NULL;
+		i->in.file = os_file_invalid;
+		i->page = NULL;
+		i->failed = FALSE;
+		return TRUE;
+	}
+
 	if (!log_online_setup_bitmap_file_range(&i->in_files, min_lsn,
 		max_lsn)) {
 
@@ -1660,6 +1777,9 @@ log_online_bitmap_iterator_next(
 		return TRUE;
 	}
 
+	if (i->end_lsn >= i->max_lsn && i->last_page_in_run)
+		return FALSE;
+
 	while (!checksum_ok)
 	{
 		while (i->in.size < MODIFIED_PAGE_BLOCK_SIZE
@@ -1752,25 +1872,32 @@ log_online_purge_changed_page_bitmaps(
 	log_online_bitmap_file_range_t	bitmap_files;
 	size_t				i;
 	ibool				result = FALSE;
+	ibool				log_bmp_sys_inited = FALSE;
 
 	if (lsn == 0) {
 		lsn = IB_ULONGLONG_MAX;
 	}
 
-	if (srv_track_changed_pages) {
+	if (srv_redo_log_thread_started) {
 		/* User requests might happen with both enabled and disabled
 		tracking */
-		mutex_enter(&log_bmp_sys->mutex);
+		log_bmp_sys_inited = TRUE;
+		mutex_enter(&log_bmp_sys_mutex);
+		if (!srv_redo_log_thread_started) {
+			log_bmp_sys_inited = FALSE;
+			mutex_exit(&log_bmp_sys_mutex);
+		}
 	}
 
-	if (!log_online_setup_bitmap_file_range(&bitmap_files, 0, lsn)) {
-		if (srv_track_changed_pages) {
-			mutex_exit(&log_bmp_sys->mutex);
+	if (!log_online_setup_bitmap_file_range(&bitmap_files, 0,
+						IB_ULONGLONG_MAX)) {
+		if (log_bmp_sys_inited) {
+			mutex_exit(&log_bmp_sys_mutex);
 		}
 		return TRUE;
 	}
 
-	if (srv_track_changed_pages && lsn >= log_bmp_sys->end_lsn) {
+	if (srv_redo_log_thread_started && lsn > log_bmp_sys->end_lsn) {
 		/* If we have to delete the current output file, close it
 		first. */
 		os_file_close(log_bmp_sys->out.file);
@@ -1778,8 +1905,20 @@ log_online_purge_changed_page_bitmaps(
 	}
 
 	for (i = 0; i < bitmap_files.count; i++) {
-		if (bitmap_files.files[i].seq_num == 0
-		    || bitmap_files.files[i].start_lsn >= lsn) {
+
+		/* We consider the end LSN of the current bitmap, derived from
+		the start LSN of the subsequent bitmap file, to determine
+		whether to remove the current bitmap.  Note that bitmap_files
+		does not contain an entry for the bitmap past the given LSN so
+		we must check the boundary conditions as well.  For example,
+		consider 1_0.xdb and 2_10.xdb and querying LSN 5.  bitmap_files
+		will only contain 1_0.xdb and we must not delete it since it
+		represents LSNs 0-9. */
+		if ((i + 1 == bitmap_files.count
+		     || bitmap_files.files[i + 1].seq_num == 0
+		     || bitmap_files.files[i + 1].start_lsn > lsn)
+		    && (lsn != IB_ULONGLONG_MAX)) {
+
 			break;
 		}
 		if (!os_file_delete_if_exists(bitmap_files.files[i].name)) {
@@ -1789,7 +1928,7 @@ log_online_purge_changed_page_bitmaps(
 		}
 	}
 
-	if (srv_track_changed_pages) {
+	if (log_bmp_sys_inited) {
 		if (lsn > log_bmp_sys->end_lsn) {
 			ib_uint64_t	new_file_lsn;
 			if (lsn == IB_ULONGLONG_MAX) {
@@ -1800,14 +1939,12 @@ log_online_purge_changed_page_bitmaps(
 				new_file_lsn = log_bmp_sys->end_lsn;
 			}
 			if (!log_online_rotate_bitmap_file(new_file_lsn)) {
-				/* If file create failed, signal the log
-				tracking thread to quit next time it wakes
-				up.  */
+				/* If file create failed, stop log tracking */
 				srv_track_changed_pages = FALSE;
 			}
 		}
 
-		mutex_exit(&log_bmp_sys->mutex);
+		mutex_exit(&log_bmp_sys_mutex);
 	}
 
 	free(bitmap_files.files);
