@@ -1,6 +1,6 @@
 /*****************************************************************************
 
-Copyright (c) 1996, 2010, Innobase Oy. All Rights Reserved.
+Copyright (c) 1996, 2015, Oracle and/or its affiliates. All rights reserved.
 Copyright (c) 2008, Google Inc.
 Copyright (c) 2009, Percona Inc.
 
@@ -26,8 +26,8 @@ ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License along with
-this program; if not, write to the Free Software Foundation, Inc., 59 Temple
-Place, Suite 330, Boston, MA 02111-1307 USA
+this program; if not, write to the Free Software Foundation, Inc., 
+51 Franklin St, Fifth Floor, Boston, MA 02110-1335 USA
 
 *****************************************************************************/
 
@@ -89,6 +89,7 @@ Created 2/16/1996 Heikki Tuuri
 # include "os0sync.h" /* for INNODB_RW_LOCKS_USE_ATOMICS */
 # include "zlib.h" /* for ZLIB_VERSION */
 # include "buf0lru.h" /* for buf_LRU_file_restore() */
+# include "os0stacktrace.h"
 
 /** Log sequence number immediately after startup */
 UNIV_INTERN ib_uint64_t	srv_start_lsn;
@@ -122,9 +123,16 @@ UNIV_INTERN enum srv_shutdown_state	srv_shutdown_state = SRV_SHUTDOWN_NONE;
 static os_file_t	files[1000];
 
 /** io_handler_thread parameters for thread identification */
-static ulint		n[SRV_MAX_N_IO_THREADS + 8];
-/** io_handler_thread identifiers */
-static os_thread_id_t	thread_ids[SRV_MAX_N_IO_THREADS + 8];
+static ulint		n[SRV_MAX_N_IO_THREADS];
+/** io_handler_thread identifiers. The extra elements at the end are allocated
+as follows:
+SRV_MAX_N_IO_THREADS + 1: srv_master_thread
+SRV_MAX_N_IO_THREADS + 2: srv_lock_timeout_thread
+SRV_MAX_N_IO_THREADS + 3: srv_error_monitor_thread
+SRV_MAX_N_IO_THREADS + 4: srv_monitor_thread
+SRV_MAX_N_IO_THREADS + 5: srv_LRU_dump_restore_thread
+SRV_MAX_N_IO_THREADS + 6: srv_redo_log_follow_thread */
+static os_thread_id_t	thread_ids[SRV_MAX_N_IO_THREADS + 7];
 
 /** We use this mutex to test the return value of pthread_mutex_trylock
    on successful locking. HP-UX does NOT return 0, though Linux et al do. */
@@ -150,7 +158,7 @@ UNIV_INTERN mysql_pfs_key_t	srv_log_tracking_thread_key;
 #endif /* UNIV_PFS_THREAD */
 
 /*********************************************************************//**
-Convert a numeric string that optionally ends in G or M, to a number
+Convert a numeric string that optionally ends in G or M or K, to a number
 containing megabytes.
 @return	next character in string */
 static
@@ -174,6 +182,10 @@ srv_parse_megabytes(
 	case 'M': case 'm':
 		str++;
 		break;
+	case 'K': case 'k':
+		size /= 1024;
+		str++;
+		break;
 	default:
 		size /= 1024 * 1024;
 		break;
@@ -617,14 +629,23 @@ open_or_create_log_file(
 		if (size != srv_calc_low32(srv_log_file_size)
 		    || size_high != srv_calc_high32(srv_log_file_size)) {
 
-			fprintf(stderr,
+			/* By default 5.6 based InnoDB will create 48M log
+			file but 5.5 only 5M. Thus give hint to user. */
+ 			fprintf(stderr,
 				"InnoDB: Error: log file %s is"
-				" of different size %lu %lu bytes\n"
-				"InnoDB: than specified in the .cnf"
-				" file %lu %lu bytes!\n",
-				name, (ulong) size_high, (ulong) size,
-				(ulong) srv_calc_high32(srv_log_file_size),
-				(ulong) srv_calc_low32(srv_log_file_size));
+ 				" of different size %lu %lu bytes\n"
+ 				"InnoDB: than specified in the .cnf"
+ 				" file %lu %lu bytes!\n",
+ 				name, (ulong) size_high, (ulong) size,
+ 				(ulong) srv_calc_high32(srv_log_file_size),
+ 				(ulong) srv_calc_low32(srv_log_file_size));
+
+			fprintf(stderr,
+				"InnoDB: Possible causes for this error:\n"
+				" (a) Incorrect log file is used or log file size is changed\n"
+				" (b) In case default size is used this log file is from 10.0\n"
+				" (c) Log file is corrupted or there was not enough disk space\n"
+				" In case (b) you need to set innodb_log_file_size = 48M\n");
 
 			return(DB_ERROR);
 		}
@@ -817,6 +838,7 @@ open_or_create_data_files(
 		}
 
 		if (ret == FALSE) {
+			const char* check_msg;
 			/* We open the data file */
 
 			if (one_created) {
@@ -914,13 +936,20 @@ open_or_create_data_files(
 				return(DB_ERROR);
 			}
 skip_size_check:
-			fil_read_first_page(
+			check_msg = fil_read_first_page(
 				files[i], one_opened, &flags,
 #ifdef UNIV_LOG_ARCHIVE
 				min_arch_log_no, max_arch_log_no,
 #endif /* UNIV_LOG_ARCHIVE */
 				min_flushed_lsn, max_flushed_lsn);
 
+			if (check_msg) {
+				fprintf(stderr,
+					"InnoDB: Error: %s in data file %s\n",
+					check_msg, name);
+				return(DB_ERROR);
+			}
+
 			if (!one_opened
 			    && UNIV_PAGE_SIZE
 			       != fsp_flags_get_page_size(flags)) {
@@ -1042,6 +1071,8 @@ skip_size_check:
 
 		if (ret == FALSE) {
 
+			const char* check_msg;
+
 			/* We open the data file */
 
 			files[i] = os_file_create(innodb_file_data_key,
@@ -1078,13 +1109,20 @@ skip_size_check:
 					(ulong) TRX_SYS_DOUBLEWRITE_BLOCK_SIZE * 9);
 			}
 
-			fil_read_first_page(
+			check_msg = fil_read_first_page(
 				files[i], one_opened, &flags,
 #ifdef UNIV_LOG_ARCHIVE
 				min_arch_log_no, max_arch_log_no,
 #endif /* UNIV_LOG_ARCHIVE */
 				min_flushed_lsn, max_flushed_lsn);
 
+			if (check_msg) {
+				fprintf(stderr,
+					"InnoDB: Error: %s in doublewrite "
+					"buffer file %s\n", check_msg, name);
+				return(DB_ERROR);
+			}
+
 			one_opened = TRUE;
 		} else {
 			/* We created the data file and now write it full of
@@ -1163,7 +1201,7 @@ init_log_online(void)
 		/* Create the thread that follows the redo log to output the
 		   changed page bitmap */
 		os_thread_create(&srv_redo_log_follow_thread, NULL,
-				 thread_ids + 5 + SRV_MAX_N_IO_THREADS);
+				 thread_ids + 6 + SRV_MAX_N_IO_THREADS);
 	}
 }
 
@@ -1238,6 +1276,28 @@ innobase_start_or_create_for_mysql(void)
 			"of memory.\n");
 	}
 
+	/* If stacktrace is used we set up signal handler for SIGUSR2 signal
+	here. If signal handler set fails we report that and disable
+	stacktrace feature. */
+
+	if (srv_use_stacktrace) {
+#if defined (__linux__) && HAVE_BACKTRACE && HAVE_BACKTRACE_SYMBOLS
+		 struct sigaction sigact;
+
+		 sigact.sa_sigaction = os_stacktrace_print;
+		 sigact.sa_flags = SA_RESTART | SA_SIGINFO;
+
+		 if (sigaction(SIGUSR2, &sigact, (struct sigaction *)NULL) != 0)
+		 {
+			 fprintf(stderr, " InnoDB:error setting signal handler for %d (%s)\n",
+				 SIGUSR2, strsignal(SIGUSR2));
+			 srv_use_stacktrace = FALSE;
+
+		 }
+#endif /* defined (__linux__) && HAVE_BACKTRACE && HAVE_BACKTRACE_SYMBOLS */
+	}
+
+
 	/* System tables are created in tablespace 0.  Thus, we must
 	temporarily clear srv_file_per_table.  This is ok, because the
 	server will not accept connections (which could modify
@@ -1594,6 +1654,7 @@ innobase_start_or_create_for_mysql(void)
 
 	fsp_init();
 	log_init();
+	log_online_init();
 
 	lock_sys_create(srv_lock_table_size);
 
@@ -2440,10 +2501,11 @@ innobase_shutdown_for_mysql(void)
 	btr_search_disable();
 
 	ibuf_close();
+	log_online_shutdown();
 	log_shutdown();
-	lock_sys_close();
 	trx_sys_file_format_close();
 	trx_sys_close();
+	lock_sys_close();
 
 	mutex_free(&srv_monitor_file_mutex);
 	mutex_free(&srv_dict_tmpfile_mutex);
