@@ -1,6 +1,7 @@
 /*****************************************************************************
 
 Copyright (c) 1996, 2011, Innobase Oy. All Rights Reserved.
+Copyright (c) 2019, MariaDB Corporation.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free Software
@@ -11,8 +12,8 @@ ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License along with
-this program; if not, write to the Free Software Foundation, Inc., 59 Temple
-Place, Suite 330, Boston, MA 02111-1307 USA
+this program; if not, write to the Free Software Foundation, Inc., 
+51 Franklin St, Fifth Floor, Boston, MA 02110-1335 USA
 
 *****************************************************************************/
 
@@ -218,9 +219,8 @@ trx_create(
 	trx_t*	trx;
 
 	ut_ad(mutex_own(&kernel_mutex));
-	ut_ad(sess);
 
-	trx = ut_malloc(sizeof(trx_t));
+	trx = mem_alloc(sizeof(trx_t));
 
 	trx->magic_n = TRX_MAGIC_N;
 
@@ -384,27 +384,6 @@ trx_allocate_for_background(void)
 	return(trx);
 }
 
-/********************************************************************//**
-Releases the search latch if trx has reserved it. */
-UNIV_INTERN
-void
-trx_search_latch_release_if_reserved(
-/*=================================*/
-	trx_t*	   trx) /*!< in: transaction */
-{
-	ulint	i;
-
-	if (trx->has_search_latch) {
-		for (i = 0; i < btr_search_index_num; i++) {
-			if (trx->has_search_latch & ((ulint)1 << i)) {
-				rw_lock_s_unlock(btr_search_latch_part[i]);
-			}
-		}
-
-		trx->has_search_latch = FALSE;
-	}
-}
-
 /********************************************************************//**
 Frees a transaction object. */
 UNIV_INTERN
@@ -467,6 +446,9 @@ trx_free(
 	ut_a(UT_LIST_GET_LEN(trx->wait_thrs) == 0);
 
 	ut_a(!trx->has_search_latch);
+#ifdef UNIV_SYNC_DEBUG
+	ut_ad(!btr_search_own_any());
+#endif
 
 	ut_a(trx->dict_operation_lock_mode == 0);
 
@@ -488,7 +470,7 @@ trx_free(
 
 	trx_release_descriptor(trx);
 
-	ut_free(trx);
+	mem_free(trx);
 }
 
 /********************************************************************//**
@@ -500,7 +482,8 @@ trx_free_prepared(
 	trx_t*	trx)	/*!< in, own: trx object */
 {
 	ut_ad(mutex_own(&kernel_mutex));
-	ut_a(trx->state == TRX_PREPARED);
+	ut_a(trx->state == TRX_PREPARED
+	     || trx->state == TRX_PREPARED_RECOVERED);
 	ut_a(trx->magic_n == TRX_MAGIC_N);
 
 	/* Prepared transactions are sort of active; they allow
@@ -526,6 +509,9 @@ trx_free_prepared(
 	ut_a(UT_LIST_GET_LEN(trx->wait_thrs) == 0);
 
 	ut_a(!trx->has_search_latch);
+#ifdef UNIV_SYNC_DEBUG
+	ut_ad(!btr_search_own_any());
+#endif
 
 	ut_a(trx->dict_operation_lock_mode == 0);
 
@@ -546,7 +532,7 @@ trx_free_prepared(
 
 	ut_ad(trx_sys->descr_n_used <= UT_LIST_GET_LEN(trx_sys->trx_list));
 
-	ut_free(trx);
+	mem_free(trx);
 }
 
 /********************************************************************//**
@@ -783,9 +769,9 @@ trx_lists_init_at_db_start(void)
 								" anyway.\n");
 
 							trx->state = TRX_ACTIVE;
-							trx_reserve_descriptor(
-								trx);
 						}
+
+						trx_reserve_descriptor(trx);
 					} else {
 						trx->state
 							= TRX_COMMITTED_IN_MEMORY;
@@ -895,6 +881,12 @@ trx_start_low(
 
 	trx->no = IB_ULONGLONG_MAX;
 
+	/* Cache the state of fake_changes that transaction will use for
+	lifetime. Any change in session/global fake_changes configuration during
+	lifetime of transaction will not be honored by already started
+	transaction. */
+	trx->fake_changes = thd_fake_changes(trx->mysql_thd);
+
 	trx->rseg = rseg;
 
 	trx->state = TRX_ACTIVE;
@@ -1158,10 +1150,11 @@ trx_commit_off_kernel(
 		lsn = 0;
 	}
 
-	ut_ad(trx->state == TRX_ACTIVE || trx->state == TRX_PREPARED);
+	ut_ad(trx->state == TRX_ACTIVE || trx->state == TRX_PREPARED
+	      || trx->state == TRX_PREPARED_RECOVERED);
 	ut_ad(mutex_own(&kernel_mutex));
 
-	if (UNIV_UNLIKELY(trx->state == TRX_PREPARED)) {
+	if (UNIV_UNLIKELY(trx->state != TRX_ACTIVE)) {
 		ut_a(trx_n_prepared > 0);
 		trx_n_prepared--;
 	}
@@ -1320,7 +1313,15 @@ trx_cleanup_at_db_startup(
 	}
 
 	trx->state = TRX_NOT_STARTED;
+
+	/* This code is executed in a single threaded context, but we acquire
+	kernel_mutex to satisfy a debug assertion in
+	trx_release_descriptor(). */
+
+	mutex_enter(&kernel_mutex);
 	trx_release_descriptor(trx);
+	mutex_exit(&kernel_mutex);
+
 	trx->rseg = NULL;
 	trx->undo_no = 0;
 	trx->last_sql_stat_start.least_undo_no = 0;
@@ -2361,6 +2362,7 @@ trx_recover_for_mysql(
 
 	while (trx) {
 		if (trx->state == TRX_PREPARED) {
+			trx->state = TRX_PREPARED_RECOVERED;
 			xid_list[count] = trx->xid;
 
 			if (count == 0) {
@@ -2385,13 +2387,25 @@ trx_recover_for_mysql(
 			count++;
 
 			if (count == len) {
-				break;
+				goto partial;
 			}
 		}
 
 		trx = UT_LIST_GET_NEXT(trx_list, trx);
 	}
 
+	/* After returning the full list, reset the state, because
+	init_server_components() wants to recover the collection of
+	transactions twice, by first calling tc_log->open() and then
+	ha_recover() directly. */
+	for (trx = UT_LIST_GET_FIRST(trx_sys->trx_list); trx;
+	     trx = UT_LIST_GET_NEXT(trx_list, trx)) {
+		if (trx->state == TRX_PREPARED_RECOVERED) {
+			trx->state = TRX_PREPARED;
+		}
+	}
+
+partial:
 	mutex_exit(&kernel_mutex);
 
 	if (count > 0){
@@ -2433,7 +2447,8 @@ trx_get_trx_by_xid(
 		the same */
 
 		if (trx->is_recovered
-		    && trx->state == TRX_PREPARED
+		    && (trx->state == TRX_PREPARED
+			|| trx->state == TRX_PREPARED_RECOVERED)
 		    && xid->gtrid_length == trx->xid.gtrid_length
 		    && xid->bqual_length == trx->xid.bqual_length
 		    && memcmp(xid->data, trx->xid.data,
